#include "DSContainer.h"
#include "DataReader.h"

DSContainer* DSContainer::me = NULL;

DSContainer* DSContainer::GetMe() {

   if(DSContainer::me != NULL) {
      return DSContainer::me;
   } else {
      cout << "creating new data set container" << endl;
      DSContainer::me = new DSContainer;
      return me;
   }
   
   
}


DSContainer::DSContainer() {
  int i;
  alldata  = vtkFieldData::New();
  alltypes = new DataReader::format[256];
  widths   = new int[256];
  heights  = new int[256];

  for(i = 0; i < 256; i++) {
     alltypes[i] = DataReader::NONE;
     widths[i]   = -1;
     heights[i]  = -1;
  }

}


DSContainer::~DSContainer() {
  cout << "deleting alldata (~DSContainer)" << endl;
  alldata->Delete();
  delete[] widths;
  delete[] heights;
  delete[] alltypes;
}

void DSContainer::Delete() {
  delete DSContainer::me;
}


void DSContainer::AddDS(vtkFloatArray* darray, DataReader::format type) {
   this->AddDS(darray, -1, -1, type); 
}


void DSContainer::AddDS(vtkFloatArray* darray, int width, int height,
                       DataReader::format type) {
  int i;
  alldata->AddArray(darray);
  alldata->GetArray(darray->GetName(),i);
  
  if(i < 256) {
    alltypes[i] = type;
    widths[i] = width;
    heights[i] = height;
  } else {
    throw MaxDataSetNumberReachedException();
    alldata->RemoveArray(darray->GetName());
  }
  
}


void DSContainer::RemoveDS(const string dsname) {
   int i;

   alldata->GetArray(dsname.c_str(),i);
   alldata->RemoveArray(dsname.c_str());

   for(int j = i; j < alldata->GetNumberOfArrays(); j++) {
      alltypes[j] = alltypes[j+1];
      widths[j] = widths[j+1];
      heights[j] = heights[j+1];
   }
   alltypes[alldata->GetNumberOfArrays()] = DataReader::NONE;
   widths[alldata->GetNumberOfArrays()] = -1;
   heights[alldata->GetNumberOfArrays()] = -1;

}

void DSContainer::RemoveDS(int idx) {
   this->RemoveDS(alldata->GetArrayName(idx));
}


void DSContainer::PrintDS(const string dsname) {
   int i;
   
   alldata->GetArray(dsname.c_str(),i);
   this->PrintDS(i);
}


void DSContainer::PrintDS(int idx) {
   vtkFloatArray* ar = vtkFloatArray::SafeDownCast(this->alldata->GetArray(idx));

   cout << "index: " << idx << endl;
   if(ar) {
      cout << "Data Set Name: " << ar->GetName() << endl;
/*      cout << "Type: ";
      switch(alltypes[idx]) {
         case DataReader::SOM:
            cout << "SOM" << endl;
            break;
         case DataReader::CLUSTER:
            cout << "CLUSTER" << endl;
            break;
         case DataReader::DATA:
            cout << "DATA" << endl;
            break;
         case DataReader::ERR:
            cout << "ERR" << endl;
            break;
         default: 
            cout << endl;
            break;
      }

      cout << "Dimension: " << ar->GetNumberOfComponents() << endl;
      cout << "# of points: " << ar->GetNumberOfTuples() << endl;
      cout << "SOM width: " << widths[idx] << endl;
      cout << "SOM height: " << heights[idx] << endl;
      cout << "Data Set: " << endl;
      for(int j = 0; j < ar->GetNumberOfTuples(); j++) {
        for(int i = 0; i < ar->GetNumberOfComponents(); i++) {
           cout << ar->GetValue(j*ar->GetNumberOfComponents() + i) << " " ;
        }
        cout << endl;
      }
      cout << endl;
 */  
   } else {
      cout << "no data set" << endl;
   }
   
}


vtkFloatArray* DSContainer::GetDS(const string dsname) {
   return vtkFloatArray::SafeDownCast(alldata->GetArray(dsname.c_str()));
}


vtkFloatArray* DSContainer::GetDS(int idx) {
  return vtkFloatArray::SafeDownCast(alldata->GetArray(idx));
}


string DSContainer::GetDSName(int id) {
   return alldata->GetArrayName(id);
}


int DSContainer::GetDSID(const string dsname) {
   int i;
   alldata->GetArray(dsname.c_str(),i);
   return i;
}


DataReader::format DSContainer::GetDSType(const string dsname) {
   int i;

   alldata->GetArray(dsname.c_str(),i);
   return alltypes[i];
}


string DSContainer::GetDSTypeAsString(const string dsname) {
   int i;
   string type;

   alldata->GetArray(dsname.c_str(),i);
   
   switch(alltypes[i]) {
     case DataReader::DATA:
        type = "DATA";
        break;
     
     case DataReader::CLUSTER:
        type = "CLUSTER";
        break;
     
     case DataReader::SOM:
        type = "SOM";
        break;

     default:
        type = "UNKNOWN";
        break;
   
   }
   
   return type;
}


DataReader::format DSContainer::GetDSType(int idx) {
   return alltypes[idx];
}

int DSContainer::GetSOMwidth(const string dsname) {
   int i;
   
   alldata->GetArray(dsname.c_str(),i);
   return widths[i];
}


int DSContainer::GetSOMwidth(int idx) {
  return widths[idx];
}


int DSContainer::GetSOMheight(const string dsname) {
  int i;
  
  alldata->GetArray(dsname.c_str(),i);
  return heights[i];
}


int DSContainer::GetSOMheight(int idx) {
  return heights[idx];
}


int DSContainer::GetNumberOfArrays() {
  return alldata->GetNumberOfArrays();
}

