#ifndef		__Normalizer
#define		__Normalizer

#ifndef VFA_H
  #define VFA_H
  #include <vtkFloatArray.h>
#endif


/// Provides some static normalization methods. 
/**
 * None of the methods allocates any memory, so make sure you initializes
 * vtkFloatArrays that are given to this methods first.
 */
class Normalizer {
  public:
    
    // methods
    
    /// Normalizes elements in range [start_idx,end_idx] of each tuple to [0,1].
    /**
     * Elements not in range [start_idx,end_idx] aren't changed. 
     * \param data Pointer to original data array.
     * \param normData Pointer to initialized array to put the normalized data into.
     * \param start_idx Lower bound for columns to use for normalization.
     * \param end_idx Upper bound for comlumns to use for normalization.
     * \param min_val Lower value for interval to normalize to. Default is 0.0.
     * \param max_val Upper value for interval to normalize to. Default is 1.0.
     */
    static void	byTuple( vtkFloatArray* data, vtkFloatArray* normData,
                         int start_idx, int end_idx, float min_val = 0.0, float max_val = 1.0 );
      
    /// Normalizes one element (at index <i>idx</i>) to [<i>min_val</i>,<i>max_val</i>].
    /** 
     * \param data Pointer to original data array.
     * \param normData Pointer to initialized array to put the normalized data into.
     * \param idx Which column/component to normalize.
     * \param min_val Lower value for interval to normalize to. Default is 0.0.
     * \param max_val Upper value for interval to normalize to. Default is 1.0.
     */
    static void oneElement(vtkFloatArray* data, vtkFloatArray* normData, int idx,
		           float min_val = 0.0, float max_val = 1.0);

     /// Normalizes elementsin range [start_idx,end_idx]  of each column.
    /** 
     * \param data Pointer to original data array.
     * \param normData Pointer to initialized array to put the normalized data into.
     * \param start_idx Lower bound for columns to use for normalization.
     * \param end_idx Upper bound for comlumns to use for normalization.
     * \param min_val Lower value for interval to normalize to. Default is 0.0.
     * \param max_val Upper value for interval to normalize to. Default is 1.0.
     */
    static void allElements(vtkFloatArray* data, vtkFloatArray* normData,int start_idx, int end_idx,
		           float min_val = 0.0, float max_val = 1.0);                             

    /// Takes all values of array into account for normalization.
    /**
     * \param data Pointer to original data array.
     * \param normData Pointer to initialized array to put the normalized data into.
     * \param start_idx Lower bound for columns to use for normalization.
     * \param end_idx Upper bound for comlumns to use for normalization.
     * \param min_val Lower value for interval to normalize to. Default is 0.0.
     * \param max_val Upper value for interval to normalize to. Default is 1.0.
     */
    static void	normAll(vtkFloatArray* data, vtkFloatArray* normData,
                        int start_idx, int end_idx, float min_val = 0.0, float max_val = 1.0 );
		
    // attributes
		
		
};
#endif
