#include "Fish.h"
#include <math.h>

#include <vtkProperty.h>
#include <vtkPolyDataMapper.h>
#include <vtkDataSetMapper.h>
#include <vtkDecimatePro.h>
#include <vtkButterflySubdivisionFilter.h>
#include <vtkTriangleFilter.h>
#include <vtkLoopSubdivisionFilter.h>
#include <vtkApproximatingSubdivisionFilter.h>
#include <vtkStripper.h>
#include <vtkSmoothPolyDataFilter.h>
#include <vtkPolyDataNormals.h>
#include <vtkImageData.h>
#include <vtkTextureMapToCylinder.h>
#include <vtkLookupTable.h>
#include <vtkPolyVertex.h>
#include <vtkSmoothPolyDataFilter.h>
#include <vtkSphereSource.h>
#include <vtkAppendPolyData.h>
#include <vtkMath.h>
#include <vtkClipPolyData.h>
#include <vtkPlaneSource.h>
#include <vtkPlane.h>
#include <vtkPointData.h>
#ifndef IOSTR
#define IOSTR
#include <iostream>
using namespace std;
#endif

int Fish::num;
const int Fish::paramnumber = 17;
const char Fish::desc[500] = "p[1],p[4]: front upper point (length,angle)\np[7],p4: rear upper point\np5,p6: upper tail fin point\np7,p[8]: front lower point \np[9],p[1]0: rear lower point\np[10],p[11]: lower tail fin point\np[1]3-p[1]5: r,g,b values\np[14]: texture, if p[14] > 0.5\np[1]7: size";
const char Fish::descfile[13] = "fishT.png";
float* Fish::paramdefaults = new float[17];
bool Fish::hsi = false;
float Fish::maxsize = 4.0;
float Fish::minsize = 4.0;
int Fish::detail = 0;

// const values for geometry



const float yLength = 2.5;

// min Winkel im 1. Quadranten - 15°
const float  minWinkel1Q= 10.0 *M_PI / 180.0;
// max Winkel im 1. Quadranten - 75°
const float  maxWinkel1Q= 80.0 *M_PI / 180.0;

// min  Winkelim 2. Quadranten - 105°
const float  minWinkel2Q= 100.0 *M_PI / 180.0;
// max  Winkelim 2. Quadranten - 165°
const float  maxWinkel2Q= 170.0 *M_PI / 180.0;

// min Winkel im 3. Quadranten - 195°
const float  minWinkel3Q= 190.0 *M_PI / 180.0;
// max  Winkelim 3. Quadranten - 255°
const float  maxWinkel3Q= 260.0 *M_PI / 180.0;

// min Winkel im 4. Quadranten - 285°
const float  minWinkel4Q= 280.0 *M_PI / 180.0;
// max Winkel im 4. Quadranten - 345°
const float  maxWinkel4Q= 350.0 *M_PI / 180.0;

const float minRadius = 0.3*yLength;

FishCellArray Fish::strips;


Fish::Fish() {
   Fish::num++;
   this->id = num;
   this->x_off = 0.0f; this->y_off = 0.0f; this->z_off = 0.0f;
   this->fLength = Fish::maxsize;
//   this->has_texture = false;
   this->debug = false;

 
   this->pdata = vtkPolyData::New();
   this->pmap = vtkPolyDataMapper::New();
   this->fishact = vtkActor::New();
  
   //   this->wholefish = vtkAssembly::New();
   
   // wholefish->AddPart( fishact );
   //   wholefish->SetPosition( x_off, y_off, z_off );
   fishact->SetPosition( x_off, y_off, z_off );
//   this->bmpread = NULL;
//   this->texture = NULL;
//   this->ptexmap = NULL;
//   this->pTransform = NULL;
   
   // initialize all parameters to 0.5 for now
   this->params = new float[Fish::paramnumber];
   
   for(int i = 0; i<Fish::paramnumber; i++) {
      this->params[i] = Fish::paramdefaults[i];
   }

  
   
}


Fish::~Fish() {
   cout << "deleting params (~Fish)" << endl;
   delete[] params;

  // points->Delete();
 //  Fish::strips->Delete();
   pdata->Delete();
 //  pmap->Delete();
   fishact->Delete();
   //   wholefish->Delete();

//   bmpread->Delete();
//   texture->Delete();
//   ptexmap->Delete();
//   pTransform->Delete();

   Fish::num--;
   cout << "Fish destructor done" << endl;
}


void Fish::SetParam( float value, int   idx ) {
  if(debug) {
    if(idx >= Fish::paramnumber) {
       cout << "void Fish::SetParam(float value, int idx): " << endl;
       cout << "Parameter index idx >= number of parameters a Fish glyph can have.";
       cout << idx << " >= " << Fish::paramnumber << "." << endl;;
    }
    if(value < 0.0 || value > 1.0) {
       cout << "void Fish::SetParam(float value, int idx): " << endl;
       cout << "Parameter value out of bounds! Must be in [0.0,1.0]" << endl;
    }
  }

  this->params[idx] = value;

/*  
  if(idx == 15) {
     this->has_texture = true;
  }
*/
 /* if(idx == 16) {
     this->fLength = (value + Fish::minsize)*(Fish::maxsize - Fish::minsize);
  }*/

}


float*	Fish::GetParams() {

   return this->params;

}

void Fish::SetColorRGB( float r, float g, float b ) {
  if(debug) {
    if(fishact == NULL) {
       cout << "void Fish::SetColorRGB(float r = 1.0, float g = 0.0, float b = 0.0 ):" << endl;
       cout << "Can't apply color because Fish has not been built yet. Call Build() first." << endl;
    }
    if( r < 0.0 || r > 1.0 || g < 0.0 || g > 1.0 || b < 0.0 || b > 1.0) {
       cout << "void Fish::SetColorRGB(float r = 1.0, float g = 0.0, float b = 0.0 ):" << endl;
       cout << "Color values must be in [0.0,1.0]." << endl;
       cout << "Currently, r = " << r << ", g = " << g << ", b = " << b << "." << endl;
    }
  }




vtkFloatArray * color = vtkFloatArray::New();
for (int i = 0; i< pdata->GetPoints ()->GetNumberOfPoints  ( );i++){
	color->InsertNextValue (0.0);
}

pdata->GetPointData()->SetScalars(color);
vtkLookupTable* table = vtkLookupTable::New();
table->SetNumberOfTableValues(3);
table-> SetTableValue(0,params[12],params[13],params[14]);
table-> SetTableValue(1,0.0,0.0,0.);
table-> SetTableValue(2,1.0,1.0,1.0);
table->Build();
pmap->SetLookupTable(table);
pmap->SetColorModeToMapScalars ();
pmap->ScalarVisibilityOn();
pmap->SetScalarModeToUsePointData ();
pmap->SetScalarRange(0.0,3.0);
  //fishact->GetProperty()->SetColor(r,g,b);
  //fishact->Modified();
}

void Fish::SetColorHSI( float H, float S, float I ) {
  if(debug) {
    if(fishact == NULL) {
       cout << "void Fish::SetColorHSI(float H, float S, float I ):" << endl;
       cout << "Can't apply color because Fish has not been built yet. Call Build() first." << endl;
    }
    if( H < 0.0 || H > 1.0 || S < 0.0 || S > 1.0 || I < 0.0 || I > 1.0) {
       cout << "void Fish::SetColorHSI(float H, float S, float I ):" << endl;
       cout << "Color values must be in [0.0,1.0]." << endl;
       cout << "Currently, H = " << H << ", S = " << S << ", I = " << I << "." << endl;
    }
  }

  H = H * 2.0 * M_PI;
  
  float r = 0.0;
  float g = 0.0;
  float b = 0.0;


  if(I == 0.0) { // black
     r = 0.0;
     g = 0.0;
     b = 0.0;
  } else if(S == 0.0) {
     r = I;
     g = I;
     b = I;
  } else {

    float P = (I/3.0)*(1.0 - S);
    float sixtydeg = 1.0/3.0 * M_PI;

    if( H <= 2.0/3.0 * M_PI ) { // < 120 degrees
      b = P;
      r = (I/3.0) * ( I + ( S * cosf(H) )/( cosf(sixtydeg - H) ) );
      g = I - (r + b);
    } else if ( H <= 4.0/3.0 * M_PI ) { // < 240 degrees
      H = H - (2.0/3.0 * M_PI );  // H - 120 degrees
      
      g = (I/3.0) * ( 1 + (S*cosf(H))/(cosf(sixtydeg - H)) );
      r = P;
      b = I - (r + g);
      
    } else { // > 240 degrees
      H = H - (4.0/3.0 * M_PI); // H - 240 degrees

      g = (I/3.0) * ( 1 + (S*cosf(H))/(cosf(sixtydeg - H)) );
      r = P;
      b = I - (r + g);
    }
  }
  

  if(debug) {
    cout << "r = " << r << endl;
    cout << "g = " << g << endl;
    cout << "b = " << b << endl;
  }

  fishact->GetProperty()->SetColor(r,g,b);
  fishact->Modified();
}


float Fish::GetColorRed() {
   return this->params[12];
}

float Fish::GetColorGreen() {
   return this->params[13];
}

float Fish::GetColorBlue() {
   return this->params[14];
}


void Fish::SetSize(float fLength, bool rebuild) {

   this->fLength = fLength;

   if(rebuild) {
     this->Build();
   }
}


float Fish::GetSize() {
   return this->fLength;
}


void Fish::SetOffset( float* xyz, bool rebuild ) {
   SetOffset(xyz[0], xyz[1], xyz[2], rebuild);
}


void Fish::SetOffset( float x, float y, float z, bool rebuild ) {
   this->x_off = x; 
   this->y_off = y;
   this->z_off = z;
   
   // this->wholefish->SetPosition( x_off, y_off, z_off );
 this->fishact->SetPosition( x_off, y_off, z_off );

   if(rebuild) {
      this->BuildRepresentation();
   }

   if(this->debug) {
      cout << "fish " << this->id << "'s offset: (" << x << ", " << y << ", " << z << ")" << endl;
      
   }
}


float Fish::GetOffsetX() {
   return this->x_off;
}


float Fish::GetOffsetY() {
   return this->y_off;
}


float Fish::GetOffsetZ() {
  return this->z_off;
}


void Fish::Build() {
   float r1,r2,r3,r4,r5,r6;
   float phi1,phi2,phi3,phi4,phi5,phi6;




 



//params[0]=0.0;
//params[1]=0.3;
   /* normalize values from [0,1] to an interval depending on the feature
    * a value is mapped to 
    */

   // für Punkt 1 - 2. Quadrant
   r1   = (params[0] * (1-minRadius) + minRadius);
   phi1 = ((1-params[1]) * (maxWinkel2Q- minWinkel2Q) + minWinkel2Q);

   // für Punkt 2 - 1. Quadrant
   r2   = (params[2] * (1-minRadius) + minRadius);
   phi2 = (params[3] * (maxWinkel1Q- minWinkel1Q) + minWinkel1Q);
 
   // für Punkt 3 - 1. Quadrant
   r3   = (params[8] * (1-minRadius) + minRadius);
   phi3 = (params[9] * (maxWinkel1Q- minWinkel1Q) + minWinkel1Q);

   // für Punkt 4- 4. Quadrant
   r4   = (params[10] * (1-minRadius) + minRadius);
   phi4 = ((1-params[11]) * (maxWinkel4Q- minWinkel4Q) + minWinkel4Q);

   // für Punkt 5- 4. Quadrant
   r5   = (params[4] * (1-minRadius) + minRadius);
   phi5 = ((1-params[5]) * (maxWinkel4Q- minWinkel4Q) + minWinkel4Q);

 // für Punkt 6- 3. Quadrant
   r6   = (params[6] * (1-minRadius) + minRadius);
   phi6 = (params[7] * (maxWinkel3Q- minWinkel3Q) + minWinkel3Q);


    



   // calculate (x,y) point positions
   x1 = r1*cos(phi1);
   y1 = r1*sin(phi1);

   x2 = r2*cos(phi2);
   y2 = r2*sin(phi2);

   x3 = r3*cos(phi3)+1.45;
   y3 = r3*sin(phi3)+0.25;

   x4 = r4*cos(phi4)+1.45;
   y4 = r4*sin(phi4)-0.25;

   x5 = r5*cos(phi5);
   y5 = r5*sin(phi5);

   x6 = r6*cos(phi6);
   y6 = r6*sin(phi6);

  
  
   this->BuildRepresentation();
}


void Fish::BuildRepresentation() {
   this->BuildPoints();
   this->BuildTriangles();
   this->BuildActors();
}




void Fish::BuildActors() {

/*
   if(has_texture) {
      if(debug) {
         cout << "Glyph " << this->id << " has a texture." << endl;
      }
      ptexmap->SetInput(this->GetData());
      cout << "texture map initialized..." << endl;
        
      pTransform->SetInput(ptexmap->GetOutput());
      pTransform->SetScale(3,3,1);
      cout << "transform initialized" << endl;
   
      bmpread->SetFileName("$FISH/../img/masonry.bmp");
      cout << "bmpread inited" << endl;

      texture->SetInput(bmpread->GetOutput());
      texture->InterpolateOn();
      cout << "applied bmp to texture" << endl;

      // \todo implement working textures...
      dsmap->SetInput( pTransform->GetOutput() );
      cout << "set transform for polydatamapper" << endl;
      
      fishact->SetMapper(dsmap);
      fishact->SetTexture(texture);

   } else {
*/


      if(Fish::hsi) {
          SetColorHSI(params[12],params[13],params[14]);
      } else {
          SetColorRGB(params[12],params[13],params[14]);
      }







float n[3];




vtkPlaneSource *planeS1 = vtkPlaneSource::New();
planeS1->SetOrigin(fLength/2*-1.5,yLength*0.025,0.0);
planeS1->SetPoint1(fLength/2*-1.05,yLength*0.4,0.0);
planeS1->SetPoint2(fLength/2*-1.05,0.0,-0.25);
planeS1->GetNormal(n);

//add the eyes
vtkSphereSource* eye1Sphere = vtkSphereSource::New();
eye1Sphere->	SetRadius (0.17);
eye1Sphere-> 	SetCenter (fLength/2*-1.1,(yLength*0.13 *y1*yLength)/2,-0.12 );
eye1Sphere->SetThetaResolution(9);
eye1Sphere->SetPhiResolution(6);
 eye1Sphere->	LatLongTessellationOn ();




vtkSphereSource* eye2Sphere = vtkSphereSource::New();
eye2Sphere->	SetRadius (0.17);
eye2Sphere-> SetCenter (fLength/2*-1.1, (yLength*0.13 *y1*yLength)/2,0.12 );
eye2Sphere->SetThetaResolution(9);
eye2Sphere->SetPhiResolution(6);
eye2Sphere->	LatLongTessellationOn ();

vtkPlane * clipPlane1 = vtkPlane::New();
clipPlane1->SetOrigin(fLength/2*-1.5,yLength*0.025,0.0);
clipPlane1->SetNormal(n[0],n[1],n[2]);

vtkClipPolyData * clipData1 = vtkClipPolyData::New();
clipData1->SetInput(eye1Sphere->GetOutput());
clipData1->SetClipFunction(clipPlane1);
clipData1->GenerateClippedOutputOn (); 

vtkPlaneSource *planeS2 = vtkPlaneSource::New();
planeS2->SetOrigin(fLength/2*-1.5,yLength*0.025,0.0);
planeS2->SetPoint1(fLength/2*-1.05,yLength*0.4,0.0);
planeS2->SetPoint2(fLength/2*-1.05,0.0,0.25);
planeS2->GetNormal(n);


vtkPlane * clipPlane2 = vtkPlane::New();
clipPlane2->SetOrigin(fLength/2*-1.5,yLength*0.025,0.0);
clipPlane2->SetNormal(n[0],n[1],n[2]);

vtkClipPolyData * clipData2 = vtkClipPolyData::New();
clipData2->SetInput(eye2Sphere->GetOutput());
clipData2->SetClipFunction(clipPlane2);
clipData2->GenerateClippedOutputOn (); 
//clipData2->InsideOutOn ();


clipData1->Update();
vtkPolyData * eye1 = clipData1->GetOutput();
vtkFloatArray * color1 = vtkFloatArray::New();
int num1 =eye1 ->GetPoints ()->GetNumberOfPoints  ( );

for (int i = 0; i<num1;i++){
	
	if (eye1 ->GetPoints ()->GetPoint(i)[2]<-0.24){ //known by trial and error
                color1->InsertNextValue (1.0);

        }
        else{
                color1->InsertNextValue (2.0);
        }

	
}
eye1->GetPointData()->SetScalars(color1);



clipData2-> Update();
vtkPolyData * eye2 = clipData2->GetClippedOutput ();
vtkFloatArray * color2 = vtkFloatArray::New();
int num2 =eye2 ->GetPoints ()->GetNumberOfPoints  ( );

for (int i = 0; i<num2;i++){	 
		   if ( eye2 ->GetPoints ()->GetPoint(i)[2]>0.24){//known by trial and error
                color2->InsertNextValue (1.0);

        }
        else {
                color2->InsertNextValue (2.0);
        }

}
eye2-> GetPointData()->SetScalars(color2);


vtkAppendPolyData*  af =vtkAppendPolyData::New();
af->AddInput(pdata);
af->AddInput(eye1);
af->AddInput(eye2);



	vtkSmoothPolyDataFilter *smooth = vtkSmoothPolyDataFilter::New();
        smooth->SetInput(af->GetOutput());
        smooth-> SetNumberOfIterations( 100);
        smooth->BoundarySmoothingOn ();
        smooth->FeatureEdgeSmoothingOn ();
	vtkTriangleFilter * triangle = vtkTriangleFilter::New();
	triangle->SetInput(smooth->GetOutput());
	triangle->GetOutput()->ReleaseDataFlagOn();
	vtkButterflySubdivisionFilter* butterfly = vtkButterflySubdivisionFilter::New();
	butterfly->SetInput(triangle->GetOutput());
	butterfly->SetNumberOfSubdivisions(Fish::detail);
	butterfly->GetOutput()->ReleaseDataFlagOn();
	/*vtkStripper *strip = vtkStripper::New();
	strip->SetInput(butterfly->GetOutput());
	strip->GetOutput()->ReleaseDataFlagOn();
	*/
      //pmap->SetInput(pdata);
	vtkPolyDataNormals* pNormals = vtkPolyDataNormals::New();
	pNormals->SplittingOn();
        //pNormals->FlipNormalsOn();
        pNormals->SetFeatureAngle (60.0);
	pNormals->SetInput(butterfly->GetOutput());
	//vtkTextureMapToCylinder * tMapPlane = vtkTextureMapToCylinder::New();
	//tMapPlane->SetInput(pNormals->GetOutput());









	pmap->SetInput(pNormals->GetOutput());
	//pmap->SetInput(pdata);
	//pmap->ImmediateModeRenderingOn();
	fishact->SetMapper(pmap);
       // fishact->SetScale(1.05);
	//fishact->GetProperty()->SetInterpolationToGouraud ();
	fishact->GetProperty()->SetInterpolationToPhong  ( ) ;
	fishact->GetProperty()->BackfaceCullingOn ( );
	/*vtkImageData* barCode = vtkImageData::New();
	barCode->SetDimensions(64,64,1);
	barCode->SetScalarTypeToUnsignedShort ();
	barCode->AllocateScalars();
	unsigned short* ptr = (unsigned short*) barCode->GetScalarPointer();
	int id =0;
	for (int i = 0; i<64;i++){
		if (i>11 && i<45&&i%2==1) id++;
	   for (int j = 0; j< 64; j++){
		if (i>11 && i<46){
		if(i%2==1&& j<(params[id]*64))
		*ptr++ =0;
		else 
		*ptr++ =1;
		
		}else 
		*ptr++ =1;
	   }
	}
	vtkTexture* barCodeText = vtkTexture:: New();
	barCodeText->SetInput(barCode);
	barCodeText-> 	InterpolateOn ();
	vtkLookupTable* table = vtkLookupTable::New();
	table->SetNumberOfTableValues(2);
	table-> SetTableValue(0,0.0,0.0,0.0);
	table-> SetTableValue(1,params[12],params[13],params[14]);
	table->Build();
*/
	//barCodeText->SetLookupTable(table);
	//fishact->SetTexture(barCodeText);


      if(debug) {
        cout << "Set color: (" << params[12] << ", " << params[13] << ", " << params[14] << ")" << endl;
      }
      
/*
  }
*/
   
}


vtkActor* Fish::GetActor() {
  return this->fishact;
}


/*vtkAssembly* Fish::GetAssembly() {
  return this->wholefish;
  }*/


vtkPolyData* Fish::GetData() {

   if(this->pdata == NULL) {
     cout << "vtkPolyData* Fish::GetData():" << endl;
     cout << "Fish has not been built yet. Try calling Build() first." << endl;
   }

   return pdata;

}


const int Fish::GetNumberOfParameters() {
   return Fish::paramnumber;
}


const char* Fish::GetParameterDescription() {
   return Fish::desc;
}


void Fish::InitNum() {
   Fish::num = 0;
}

int Fish::GetNumberOfFishes() {
   return Fish::num;
}

int Fish::GetID() {
   return this->id;
}

void Fish::SetID( int id ) {
   this->id = id;
}


void Fish::BuildPoints() {

/* insert all 3d points */


vtkPoints * points = vtkPoints::New();



 //2d fish points
  points->InsertNextPoint(fLength/2*-1.5,yLength*0.025,0.0); // 0
  points->InsertNextPoint(fLength/2*-1.05,(yLength*0.4*yLength*y1)/2,0.0); //1
  points->InsertNextPoint(fLength/2*-1.05,0.0,-0.25); //29
  points->InsertNextPoint(fLength/2*-1.05,0.0,0.25); //30

 // Point 2- first mapped point (above fish)
 points->InsertNextPoint(fLength/2*x1,yLength*y1,0.0);//2
  points->InsertNextPoint(fLength/2*0.0,0.0,0.35); //31
  points->InsertNextPoint(fLength/2* 0.0,0.0,-0.35); // 32
  points->InsertNextPoint(fLength/2*((x2-x1)*0.65+x1),yLength*((y1+y2)/2+1.25*params[15]),0.0); //3
 
  // Point 8 - second mapped point (above fish)
 points->InsertNextPoint(fLength/2*x2,yLength*y2,0.0);//8
   points->InsertNextPoint(fLength/2*(((x3-x2)*0.6+x2)+((x4-x5)*0.6+x5))/2,yLength*0.2,0.0);//9

 // Point 11 - tip of fish's tail (third mapped point)
  points->InsertNextPoint(fLength/2*x3,yLength*y3,0.0); //11
 points->InsertNextPoint(fLength/2*((x3+x4)/2-0.2),0.0,0.0); //12
  // Point 13 - fourth mapped point (below fish)
 
  points->InsertNextPoint(fLength/2*x4,yLength*y4,0.0);//14
  points->InsertNextPoint(fLength/2*(((x3-x2)*0.6+x2)+((x4-x5)*0.6+x5))/2,yLength*-0.2,0.0);
// Point 16 - fifth mapped point (below fish)
 points->InsertNextPoint(fLength/2*x5,yLength*y5,0.0); //16
 
 //int p18 = points->InsertNextPoint(1*(x5-(x5-x6)*0.15),(y5+y6)/2-0.7,0.0); //18
 
 //int p20 =  points->InsertNextPoint(1*(x5-(x5-x6)*0.5),(y5+y6)/2,0.0); //20
// int p21 =  points->InsertNextPoint(1*(x5-(x5-x6)*0.7),(y5+y6)/2,0.0);//21
 points->InsertNextPoint(fLength/2*(x5-(x5-x6)*0.35),yLength*((y5+y6)/2-1.25*params[16]),0.0); //22
 
 
   // Point 26 - tip of fish's lower tail - sixth mapped point (below fish)
   points->InsertNextPoint(fLength/2*x6,yLength*y6,0.0); // 26
  points->InsertNextPoint(fLength/2*-1.05,(yLength*0.4*yLength*y6)/2,0.0); // 27
points->InsertNextPoint(fLength/2*-1.5,yLength*-0.025,.0); // 28
  

   //3d fish points
 
 
 
 points->InsertNextPoint( fLength/2*((x2-x1)*0.65+x1),yLength*((y1+y2+0.075)/2),0.04); //34
 points->InsertNextPoint( fLength/2*((x4-x5)*0.65+x5),0.0,0.15); //35
  points->InsertNextPoint(fLength/2*((x3-x2)*0.65+x2),0.0,-0.15); //36
  points->InsertNextPoint( fLength/2*((x2-x1)*0.65+x1),yLength*((y1+y2+0.075)/2),-0.04); //37
// int p38 = points->InsertNextPoint(1*( x5-(x5-x6)*0.2),(y5+y6)/2,0.-0.02); // 38
points->InsertNextPoint( fLength/2*(x5-(x5-x6)*0.35),yLength*(y5+y6)/2,-0.02); // 39
 //int p24 = points->InsertNextPoint(1*(x5-(x5-x6)*0.2),(y5+y6)/2,0.02); //24
  points->InsertNextPoint(fLength/2*(x5-(x5-x6)*0.35),yLength*(y5+y6)/2,0.02); // 25




 pdata->SetPoints(points);
  
  

  
}


void Fish::BuildTriangles() {


  pdata->SetStrips(Fish::strips.getCellArray());
  

pdata->BuildLinks();
}   


void Fish::SetDefaults(float p0, float p1, float p2, float p3,  float p4, float p5,
                       float p6, float p7, float p8, float p9, float p10, float p11,
                       float rh, float gs, float bi, float texture, float size) {
   Fish::paramdefaults[0] = p0;
   Fish::paramdefaults[1] = p1;
   Fish::paramdefaults[2] = p2;
   Fish::paramdefaults[3] = p3;
   Fish::paramdefaults[4] = p4;
   Fish::paramdefaults[5] = p5;
   Fish::paramdefaults[6] = p6;
   Fish::paramdefaults[7] = p7;
   Fish::paramdefaults[8] = p8;
   Fish::paramdefaults[9] = p9;
   Fish::paramdefaults[10] = p10;
   Fish::paramdefaults[11] = p11;
   Fish::paramdefaults[12] = rh;
   Fish::paramdefaults[13] = gs;
   Fish::paramdefaults[14] = bi;
   Fish::paramdefaults[15] = texture;
   Fish::paramdefaults[16] = size;
}

void Fish::SetDefaults(float* params) {
   int i;
   
   for(i = 0; i < paramnumber; i++) {
      Fish::paramdefaults[i] = params[i];
   }

}

void Fish::EnableHSI( bool hsi ) {
   
   Fish::hsi = hsi;

}

bool Fish::GetHSI() {
   return Fish::hsi;
}

float Fish::GetMaxSize() {
   return Fish::maxsize;
}

float Fish::GetMinSize() {
   return Fish::minsize;
} 

void Fish::SetMaxSize( int max_size ) {
   Fish::maxsize = max_size;
}

void Fish::SetMinSize( int min_size ) {
   Fish::minsize = min_size;
}

float* Fish::GetDefaults() {
   return Fish::paramdefaults;
}

void Fish::SetLevelOfDetail(int newDetail){
	Fish::detail = newDetail;	
}

