#include <time.h>
#include "FishCloud.h"
#include "vtkPolyData.h"
#include "vtkProperty.h"
#include "Renderer.h"
#include "vtkTextActor.h"

// TODO: write interfaces for Glyph and GlyphCloud...


FishCloud::FishCloud( vtkFloatArray* darray, int*  mapping, bool som,
                      float x_off, float y_off, float z_off) {

   this->x_off = x_off;
   this->y_off = y_off;
   this->z_off = z_off;
   this->ost = RANDOM; // set offsetType
   this->darray = darray;
   this->som = som;
   this->cloudsize = darray->GetNumberOfTuples();
   this->radmax = 0.0;// war bis jetz nicht initalisiert und wird in der Funktion nicht immer gesetzt -> netter Bugs mit zufaelligen Radien 

   /* this->sphereact = vtkActor::New();
   this->sphere = vtkSphereSource::New();
   this->spheremap = vtkPolyDataMapper::New();
   */
   this->debug = false;
   this->default_height = Renderer::GetDefaultHeight();
   
   fishes = (Fish**)malloc(sizeof(Fish *) * cloudsize );
   for(int i = 0; i < cloudsize; i++) {
      fishes[i] = new Fish;
   }


   GenerateOffsets();
   SetMapping(mapping);   

   fishacts = vtkActorCollection::New();
   for(int i = 0; i < cloudsize; i++) {
     fishacts->AddItem(fishes[i]->GetActor());
     fishes[i]->GetActor()->SetPosition( x_off, y_off, z_off );
   }

   
    //fishes[0]->GetActor()->GetProperty()->SetOpacity(0.4);
   //   fishacts->SetPosition( x_off, y_off, z_off );

   
  /* if(debug) {
     for( int i = 0; i < darray->GetNumberOfTuples(); i++ ) {
       for( int j = 0; j < darray->GetNumberOfComponents(); j++) {
         cout << darray->GetValue( i*darray->GetNumberOfComponents() + j ) << "   ";
       }
       cout << endl;
     }
   }
   */
}


FishCloud::~FishCloud() {
   
  // sphereact->Delete();
  //spheremap->Delete();
  //sphere->Delete();
  
  fishacts->Delete();
  cout << "deleted fishact (~FishCloud)" << endl;

  for(int i = 0; i < cloudsize; i++) {
    delete fishes[i];
  }
  free(fishes);
  cout << "deleted fishes (~FishCloud)" << endl;

}


vtkActorCollection* FishCloud::GetActors() {

  return fishacts;

}


Fish* FishCloud::GetSingle(int idx) {

   if(idx < this->cloudsize && idx >= 0) {
     return this->fishes[idx];
   } 
    
   return NULL;

}

void FishCloud::Build() {
   int i;
   for(i = 0; i < cloudsize; i++) {
      fishes[i]->Build();
   }

   if(cloudsize != 1) {
     /* sphere->SetCenter(x_off,y_off,z_off);
      sphere->SetRadius(0.3);
      spheremap->SetInput( sphere->GetOutput() );
      sphereact->SetMapper( spheremap );
      sphereact->GetProperty()->SetColor( 0.0,0.0,0.0);
      //sphereact->GetProperty()->SetOpacity(0.4);
      sphereact->Modified();
     */
    
   }
   
}


void FishCloud::Rebuild(bool all) {
   int i;

   if(all) {
      for(i = 0; i < cloudsize; i++) {
         fishes[i]->Build();
      }      
   } else {
      for(i = 0; i < cloudsize; i++) {
         fishes[i]->BuildRepresentation();
      }
   }
   
   if(cloudsize != 1) {
     // sphere->SetCenter(x_off, y_off, z_off);
     //  sphereact->Modified();
   
   }

}


void FishCloud::SetOffset( float x, float y, float z, bool done ) {
   this->x_off = x;
   this->y_off = y;
   this->z_off = z;

   if(done) {
      GenerateOffsets();
   }
}


void FishCloud::SetOffset( float* off, bool done ) {
   this->SetOffset(off[0], off[1], off[2], done);
}


float FishCloud::GetOffsetX() {
   return this->x_off;
}


float FishCloud::GetOffsetY() {
   return this->y_off;
}


float FishCloud::GetOffsetZ() {
   return this->z_off;
}


int FishCloud::GetNumberOfGlyphs() {
   return this->cloudsize;
}

void FishCloud::SetSize(float mult, int idx) {
   int i;

   if(idx == -1) {
      for(i=0 ; i < this->cloudsize ; i++) {
         fishes[i]->SetSize(mult);
      }
   } else {
      fishes[idx]->SetSize(mult);
   }
}

float FishCloud::GetSize(int idx) {
   // since all glyphs in a cloud have to have the same size,
   // and clouds can't have less than one glyph member, 
   // we take the size from the first fish in the cloud:
   return fishes[idx]->GetSize();
}

void FishCloud::SetMapping( int* mapping ) {
   int i,j;

   if(this->debug) {
      cout << " cloudsize: " << cloudsize << endl;
      cout << " size of darray: " << darray->GetNumberOfTuples() << endl;
   }
   
   for(i = 0; i < cloudsize; i++) {

      for(j = 0; j < Fish::GetNumberOfParameters(); j++) {
         if(mapping[j] != -1) {
            fishes[i]->SetParam( darray->GetComponent(i,mapping[j]), j);
            if(debug) {
               cout << i << ": " << darray->GetComponent(i,mapping[j]) << ", which is the " << mapping[j] << "th component, has been mapped to feature " << j << "." << endl;
            }
         }
      }
      

   }
}


void FishCloud::SetDataArray(vtkFloatArray* darray) {
   this->darray = darray;
}


void FishCloud::SetSOM( bool som, bool done ) {
   this->som = som;
  
   if( done ) {
      GenerateOffsets();
   }
}

void FishCloud::GenerateOffsets() {
   int i;
   int fish_is_set;
   float x,y,z;
   float phi, theta, r;

   if(cloudsize > 0) {
     this->radmax =  powf( (float)cloudsize,(1.0f/3.0f) ) * Fish::GetMaxSize();
     if(debug) {
       cout << "radmax = " << radmax << endl;
     }

     srand((int)time(NULL)+fishes[0]->GetID()); // initialize random number generator
     if(cloudsize > 1) {
       switch(ost) {

         case RANDOM:
           for(i = 0; i < cloudsize; i++) {

              fish_is_set = 0;
              x = 0.0; 
              y = 0.0;
              z = 0.0;

              do {

                 phi = ( (float)rand()   * 2*M_PI )/RAND_MAX;
                 theta = ( (float)rand() * 2*M_PI )/RAND_MAX;
                 r = powf( (((float)rand()/RAND_MAX) * powf(this->radmax,3.0f)), (1.0f/3.0f) );
 
                 if(debug) {
                   cout << "r = " << r << endl;
                 }

                 x = r * cos(phi);
                 if(this->som) {
                    y = default_height + r * sin(phi) * sin(theta);
                 } else {
                    y = r * sin(phi) * sin(theta);
                 } 
                 z = r * sin(phi) * cos(theta);
                 if(this->debug) {
                    cout << "rand position: (" << x << ", " << y << ", " << z << ")" << endl;
                 }
              } while( DetectCollision(x+x_off,y+y_off,z+z_off) );
           
              fishes[i]->SetOffset( x_off+x, y_off+y, z_off+z );                    
           }

           break;

         // end of case RANDOM ////////////////////



         case DISTANCE:   
           // For implementing this, the FishCloud class needs to know of 
           // the prototype vectors or SOM nodes somehow.
           // Maybe it should be given a list of distances...
           cout << "Distance calculated fish clouds aren't implemented yet." << endl;
           break;
       
         default:
           // should not occur. If it does, all fishes will have
           // the same position...
           cout << "oops, this should not have happened... No offset type was given to this cloud." << endl;
           exit(1);
       

       }

     } else {  // if cloudsize == 1, don't change positions
       fishes[0]->SetOffset( x_off, y_off, z_off );
     }
  } // fi cloudsize > 0

}



// TODO: SetDistanceArray(float* distances); (setzt r, Winkel zufaellig errechnet)


int FishCloud::DetectCollision(float x, float y, float z) {
   int i;
   float length;

   for(i = 0; i < cloudsize; i++) {
      length = fishes[i]->GetSize();
      // test if the new position would intersect another fish.
      if(fishes[i]->GetOffsetX() <= x + length &&
         fishes[i]->GetOffsetX() + length >= x &&
         fishes[i]->GetOffsetY() - length/2.0f <= y + length/2.0f &&
         fishes[i]->GetOffsetY() + length/2.0f >= y - length/2.0f &&
         fishes[i]->GetOffsetZ() - length/2.0f <= z + length/2.0f &&
         fishes[i]->GetOffsetZ() + length/2.0f >= z - length/2.0f) {
         if(debug) {
            cout << "collision detected: Offset of fishes[" << i << "] = (" << fishes[i]->GetOffsetX() << ", " << fishes[i]->GetOffsetY() << ", " << fishes[i]->GetOffsetZ() << "). (x,y,z) = " << x << ", " << y << ", " << z << ")" << endl;
         }

         return 1;
      } 
   }

   return 0;

}


void FishCloud::SetMaxRadius(float r) {
   this->radmax = r;
}

float FishCloud::GetMaxRadius() {
   return this->radmax;
}
