#include "GlyphWrapper.h"


const bool GlyphWrapper::modeline[1] = {true};   // [ FISH: true, <other glyphs> ... ]
                                    // For FISH it's true because the
                                    // FISH Glyph has a cloud mode.


GlyphWrapper::GlyphWrapper(enum glyphType type, 
                           vtkFloatArray* darray, int* mapping, bool som,
                           float x_off, float y_off, float z_off, bool single) {

   this->debug = true;
   this->type = type;
   this->single_mode = single;

   this->fishes = NULL;
/*
   Fish::SetDefaults( 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f,
                      0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f,
                      0.0f, 1.0f, 0.8f,
                      0.0f, 1.0f);
   Fish::EnableHSI(true);    
*/                  
   // You should set variables you declared for your new glyph to NULL here
   // so the destructor can check if they have been initialized.
   // also set default values here

   switch(type) {
      
       /* Section for Fish glyphs */
       case FISH:
               
         fishes = new FishCloud(darray, mapping, som, x_off, y_off, z_off );
         break;
       /* --- End of Fish glyph section --- */
      
      
       /* --- insert your own glyph sections here: ---
       case NEW_GLYPH:
         glyph_var_name = new NewGlyph;
         break;
      
       case ANOTHER_GLYPH:      
         another_glyphs_var_name = new AnotherGlyph;
         break;
      
       ...etc.
       --- End of own sections --- */
         
       default:
          cout << "GlyphWrapper::GlyphWrapper(enum glyphType type, bool single): No type was given for this Glyph." << endl;
          break;
   }
/*   
   data = vtkFloatArray::New();
   data->SetNumberOfComponents( darray->GetNumberOfComponents() );
   data->SetNumberOfValues( darray->GetNumberOfComponents() * darray->GetNumberOfTuples() );
   data->DeepCopy( darray );
*/  
}


GlyphWrapper::~GlyphWrapper() {
   if(fishes != NULL) {
      delete fishes;
   }
   cout << "called delete fishes (~GlyphWrapper)" << endl;
   
//   data->Delete();
   
   // same for other glyphs variables
}




void GlyphWrapper::SetParams( vtkFloatArray* darray, int* mapping, bool som, 
                                   float x_off, float y_off, float z_off ) {
   
   switch(this->type) {
      
      /* --- Fish section */
      case FISH:

        if(fishes == NULL) {
           fishes = new FishCloud(darray, mapping, som, x_off, y_off, z_off);
        } else {
           fishes->SetDataArray(darray);
           fishes->SetMapping(mapping);
           fishes->SetSOM(som, false); 
           fishes->SetOffset(x_off, y_off, z_off);
           fishes->Rebuild(true); // rebuilds everything anew
        }
        /* --- end of Fish setion */
        
        /* your glyphs implementation of renewing params goes here */
        break;
        
      default:
         cout << " GlyphWrapper::SetParams( vtkFloatArray* darray, int* mapping, bool som, float x_off, float y_off, float z_off ): " << endl;
         cout << "No type was given for this glyph." << endl;
         break;
   }

}


// TODO:: implement this method! (SetGlyphType)
void GlyphWrapper::SetGlyphType( enum glyphType new_type, bool single ) {

   if(this->type != new_type) {
      glyphType old = this->type;
      this->type = new_type;
      this->single_mode = single;

      switch(this->type) {
      
         /* --- Fish section */
         case FISH:
            // TODO: implement type change when the first new glyph class comes in.
            // We have to transfer the data array pointer, the mapping pointer, som mode,
            // and offset values to the newly created instance.          
            break;
         /* --- end of Fish setion */


         default: 
            cout << "blah" << endl;
            break;
      }
      
   }
}


void GlyphWrapper::SetMapping( int* mapping ) {

   switch(this->type) {
      
      /* --- Fish section */
      case FISH:

        if(fishes == NULL) {
           cout << "can't set mapping without having initilized your glyph instance" << endl;
           exit(1);
        } else {
           fishes->SetMapping(mapping);
           fishes->Rebuild(true); // rebuilds everything anew
        }
        break;
        /* --- end of Fish setion */
        
        /* your glyphs implementation of renewing params goes here */
        
      default:
         cout << " GlyphWrapper::Mapping( int* mapping ): No type was given for this glyph." << endl;
         break;
   }


}


void GlyphWrapper::SetSingleParam( float value, int  idx ) {
   if(!single_mode) {
      if(debug) {
         cout << "void GlyphWrapper::SetSingleParam( int* mapping ):" << endl;
         cout << "You can't call a single glyph method when in cloud mode!!!" << endl;
         cout << "From now on everything can happen!" << endl;
      }
   } else {
   
      switch(this->type) {

         /* --- Fish section --- */     
         case FISH:
            fishes->GetSingle(0)->SetParam(value, idx);
            break;
         /* --- end of Fish section */



         // case OTHER_GLYPH:
         //    your implementation here



         default:
            cout << "GlyphWrapper::SetSingleParam(float value, int idx): No type was given for this Glyph." << endl;
            break;

      }      
   }
}




float* GlyphWrapper::GetSingleParams() {

   if(!single_mode) {
      if(debug) {
         cout << "float* GlyphWrapper::GetSingleParams():" << endl;
         cout << "You can't call a single glyph method when in cloud mode!!!" << endl;
         cout << "From now on everything can happen!" << endl;
      }
   } else {
   
      switch(this->type) {

         /* --- Fish section --- */     
         case FISH:
            return(fishes->GetSingle(0)->GetParams());
            break;
         /* --- end of Fish section */



         // case OTHER_GLYPH:
         //    your implementation here
         //    ...
         //    return( blah );
         //    break;


         default:
            cout << "float* GlyphWrapper::GetSingleParams(): No type was given for this glyph." << endl;
            break;

      }      
   }
   
   return NULL;

}





void GlyphWrapper::Build() {
   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
         fishes->Build();
         break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLYPH
      //    your implementation here;
      //    break;

      default:
         cout << "GlyphWrapper::Build(): No type was given for this glyph." << endl;
         break;
   
   }
}



void GlyphWrapper::Rebuild(bool all) {

   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
        if(all) {
           fishes->Rebuild(true);
        } else {  
           fishes->Rebuild(false);
        }   
        break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLYPH
      //    your implementation here;
      //    break;

      default:
         cout << "GlyphWrapper::Rebuild(bool all): No type was given for this glyph." << endl;
         break;
   }   
}


void GlyphWrapper::SetSize( float size, int idx ) {
   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
         fishes->SetSize(size, idx);
         break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLYPH
      //    your implementation here;
      //    break;


      default:
         cout << "Help! Something unforseen happened!" << endl;
         break;

   }
}



float GlyphWrapper::GetSize(int idx) {

   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
         return( fishes->GetSize(idx) );
         break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLYPH
      //    your implementation here;
      //    size_dummy = blah;
      //    break;


      default:
         cout << "Help! Something unforseen happened!" << endl;
         break;

   }

   return(-1);
}


int GlyphWrapper::GetCloudCount() {

   switch(this->type) {

      /* --- Fish section --- */     
      case FISH:
         return( fishes->GetNumberOfGlyphs() );
         break;
      /* --- end of Fish section */



      // case OTHER_GLYPH:
      //    your implementation here
      //    ... 
      //    return( blah );
      //    break;



      default:
         cout << "int GlyphWrapper::GetCloudCount(): No type was given for this glyph." << endl;
         break;
           
   }
   
   return(-1);
   
}


void GlyphWrapper::SetOffset( float* xyz ) {
   this->SetOffset(xyz[0], xyz[1], xyz[2]);

}


void GlyphWrapper::SetOffset( float x, float y, float  z ) {
   
   // TODO: Recalculate positions of all Glyphs and update actors! so this can be used during rendering

   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
         fishes->SetOffset(x,y,z);
         break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLYPH
      //    your implementation here;
      //    break;


      default:
         cout << "GlyphWrapper::SetOffset(float x, float y, float z): No type was given for this glyph." << endl;
         cout << "type: " << (int)this->type << endl;
         break;

   }

}



float GlyphWrapper::GetOffsetX() {

   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
         return( fishes->GetOffsetX() );
         break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLYPH
      //    your implementation here;
      //    break;


      default:
         cout << "float GlyphWrapper::GetOffsetX(): No type was given for this glyph." << endl;
         break;
       
       
   }

   return(1000000.0);

}



float GlyphWrapper::GetOffsetY() {

   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
         return( fishes->GetOffsetY() );
         break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLYPH
      //    your implementation here;
      //    break;


      default:
         cout << "float GlyphWrapper::GetOffsetY(): No type was given for this glyph." << endl;
         break;
       
       
   }

   return(1000000.0);

}



float GlyphWrapper::GetOffsetZ() {

   switch(this->type) {
   
      /* --- Fish section --- */
      case FISH:
         return( fishes->GetOffsetZ() );
         break;
      /* --- end of Fish section --- */
      
      
      // case OTHER_GLZPH
      //    your implementation here;
      //    break;


      default:
         cout << "float GlyphWrapper::GetOffsetZ(): No type was given for this glyph." << endl;
         break;
       
       
   }

   return(1000000.0);

}


/* TODO: implement GetData
vtkPolyData** GlyphWrapper::GetData() {
}
*/



vtkActorCollection* GlyphWrapper::GetActors() {

      switch(this->type) {

         /* --- Fish section --- */     
         case FISH:
            return ( fishes->GetActors() );
            break;
         /* --- end of Fish section */



         // case OTHER_GLYPH:
         //    your implementation here
         //    ... 
         //    dummy = blah;
         //    break;



         default:
            cout << "vtkActor** GlyphWrapper::GetActorArray(): No type was given for this glyph." << endl;
            break;
   }
   
   return NULL;
}


int GlyphWrapper::GetNumberOfParameters() {

   return GlyphWrapper::GetNumberOfParameters(this->type);

}


int GlyphWrapper::GetNumberOfParameters( enum glyphType type ) {

   switch(type) {
      case FISH:
         return( Fish::GetNumberOfParameters() );
         break;

      // other glyphs go here

      default: 
         cout << "GlyphWrapper::GetNumberOfParameters(enum glyphType type): No type was given for this glyph." << endl;
         break;
   }

   return(-1);
}


const string GlyphWrapper::GetParameterDescription( enum glyphType type ) {

   switch(type) {
      case FISH:
         return( Fish::GetParameterDescription() ); 
         break;

        // other glyphs go here

      default: 
         cout << "GlyphWrapper::GetParameterDescription(enum glyphType type): No type was given for this glyph." << endl;
         break;
   }
   
   return NULL;
}


void GlyphWrapper::SetMaxRadius(float r) {
   switch(type) {
      case FISH:
         fishes->SetMaxRadius(r);
         break;
         
      default:
         cout << "GlyphWrapper::SetMaxRadius(float r): No type was given for this glyph." << endl;
         break;
   }
   
}  


float GlyphWrapper::GetMaxRadius() {
   switch(type) {
      case FISH:
         return fishes->GetMaxRadius();
         break;
         
      default:
         cout << "GlyphWrapper::SetMaxRadius(float r): No type was given for this glyph." << endl;
         break;
         
   }
   
   return -1;
   
}  


/* TODO: implement textures correctly!
vtkTexture* GlyphWrapper::GetTexture() {
}
*/


const string GlyphWrapper::GetDescFile(int gtype) {
   switch(gtype) {
      case FISH:
         return Fish::descfile;
         break;
         
      default:
         cout << "GlyphWrapper::GetDescFile(): No type was given for this glyph." << endl;
         break;
         
   }
   
   return "";
}


void GlyphWrapper::SetDefaults(float* params, glyphType type) {

   switch( type ) {
      case FISH:
         Fish::SetDefaults( params );
         break;
     
      default:
         cout << "GlyphWrapper::SetDefaults(): No type was given for this glyph" << endl;
         break;
   }

}



float* GlyphWrapper::GetDefaults() {
   switch( type ) {
      case FISH:
         return Fish::GetDefaults();
         break;

      default:
         cout << "GlyphWrapper::GetDefaults(): No type was given for this glyph" << endl;
         break;
    
   }
   
   return NULL;
}


void GlyphWrapper::EnableHSI( bool enable ) {
   
   switch( type ) {
      case FISH:
         Fish::EnableHSI( enable );
         break;
         
      default:
         cout << "GlyphWrapper::EnableHSI(): No type was given for this glyph" << endl;
         break;
         
   }
   
}


bool GlyphWrapper::GetHSI() {
   
   switch( type ) {
      case FISH:
         return Fish::GetHSI();
         break;
         
      default:
         cout << "GlyphWrapper::GetHSI(): No type was given for this glyph" << endl;
         break;
      
   }
   
   return true;
   
}



void GlyphWrapper::EnableHSI( bool enable, glyphType type ) {
   
   switch( type ) {
      case FISH:
         Fish::EnableHSI( enable );
         break;
         
      default:
         cout << "GlyphWrapper::EnableHSI(): No type was given for this glyph" << endl;
         break;
         
   }
   
}


bool GlyphWrapper::GetHSI( glyphType type ) {
   
   switch( type ) {
      case FISH:
         return Fish::GetHSI();
         break;
         
      default:
         cout << "GlyphWrapper::GetHSI(): No type was given for this glyph" << endl;
         break;
      
   }
   
   return true;
   
}



