#include "Grid.h"
#include "Fish.h"
#include "Normalizer.h"
#include "Settings.h"
#include <vtkFloatArray.h>
#include <vtkCellArray.h>
#include <vtkPolyData.h>
#include <vtkDataSet.h>
#include <vtkPointData.h>
#include <vtkProperty.h>
#include <vtkImageData.h>
#include <vtkWarpScalar.h>
#include <vtkImageDataGeometryFilter.h>
#include <vtkMergeFilter.h>
#include <vtkDataSetMapper.h>
#include <vtkActorCollection.h>
#include <vector.h>
#include <vtkPolyDataNormals.h>
#include <vtkJPEGReader.h>
#include <vtkTexture.h>
#include <vtkPlaneSource.h>
#include <vtkTextureMapToPlane.h>
#include <vtkDataSetMapper.h>
#include <vtkPolygon.h>
#include <vtkQuad.h>
#include <vtkPolyVertex.h>
#include <vtkSphereSource.h>
#include <vtkTextureMapToCylinder.h>
#include <vtkTextureMapToSphere.h>
#include <vtkButterflySubdivisionFilter.h>
#include <vtkTriangleFilter.h>
#include <vtkProperty.h>


Grid::Grid() : QObject() {
  mode = 0;
  node_dist = 1.0;
  height = 1.0;
  xnum = znum = -1;
   

  pLUT = vtkLookupTable::New();

  gridActorCollection = vtkActorCollection::New();
  //  heights = vtkFloatArray::New();

  som = NULL;
 
   
}

Grid::~Grid() {

  for (int i = znum-1; i >= 0; i--)
    {
      delete []  heights[i];
    }
  delete [] heights;
gridActorCollection->Delete();
}

void Grid::SetDim( int x, int z ) {
   this->xnum = x;
   this->znum = z;


   heights = new (float*[znum]);
   for (int j = 0; j <znum; j++){
     heights[j] = new float[xnum];
   }

}




void Grid::SetMode( int mode ) {
   this->mode = mode;
}


void Grid::SetComponent( int component ) {
   this->component = component;
}


void Grid::SetNodeDist( float dist ) {
   this->node_dist = dist;
}


void Grid::SetHeight( float height) {
   this->height = height;
}


vtkActorCollection* Grid::Build() {
   float shift = Fish::GetMinSize()/2; // shift grid a little, because offsets of 
                                       // fishes are at the snout.
  gridActorCollection->RemoveAllItems ();

   cout << "inside Build method" << endl;


  vtkPoints* pPoints = vtkPoints::New();
 // vtkFloatArray* pFloats = vtkFloatArray::New();
  vtkPolyData *pData = vtkPolyData::New();
  vtkDataSetMapper* pMap = vtkDataSetMapper::New();
  vtkActor* gridActor = vtkActor::New();
  vtkCellArray* pCells = vtkCellArray::New();
  pPoints->SetNumberOfPoints( xnum*znum );
  //pFloats->SetNumberOfValues( xnum*znum );
  // pLUT->SetHueRange( 0.65, 0.50);
  pLUT->SetHueRange(  0.50,0.65);

  //  pLUT->SetNumberOfColors (256);
  //  pLUT->SetNumberOfTableValues(256);    
  /* for (int i = 255; i >= 0; i--) {
    float r =(float)i/(255*2)+0.5f;
    float b =(float)i/(255*2)+0.5f;
    float g  =(float)i/(255*2)+0.5f;
    cout << r <<" "<<g<<" "<<b<< endl;
    pLUT->SetTableValue(i,r,g,b);
    }  */


vtkFloatArray * values;
//int component = getComponent();

   cout << "mode: " << mode << endl;
   switch(mode) {
     float tmppnt[3];


     case Settings::clustering: 	// component plane
       // pMap->ScalarVisibilityOff();
        values = cluster;
	if (values!= NULL){
		pLUT->SetTableRange(cluster->GetRange());
		for( int j = 0; j < znum; j++ ) {
			for( int i = 0; i < xnum; i++) {
				tmppnt[0] = i * node_dist + shift;
				tmppnt[1] = (1-values->GetValue(j*xnum+i)) *height;
				heights[j][i]=tmppnt[1];
				tmppnt[2] = j * node_dist;
				pPoints->SetPoint(j*xnum+i,tmppnt);
				// pLUT->SetHueRange(0.0,1.0);
			}
		}
	}
        break;



     case Settings::componentPlane: 	// component plane
       // pMap->ScalarVisibilityOff();
        values = vtkFloatArray::New();
        for( int j = 0; j < znum; j++ ) {
          for( int i = 0; i < xnum; i++) {
            tmppnt[0] = i * node_dist + shift;
	    values->InsertNextValue(som->GetTuple(j*xnum+i)[component]);
            tmppnt[1] = (1-som->GetTuple(j*xnum+i)[component])*height;
	    heights[j][i]=tmppnt[1];
            tmppnt[2] = j * node_dist;
            pPoints->SetPoint(j*xnum+i,tmppnt);
          }
        }
        break;

     case Settings::umat:	// unified distance matrix
     values =  Umat(this->som);
	//  pMap->ScalarVisibilityOn();
        for( int j = 0; j < znum; j++ ) {
          for( int i = 0; i < xnum; i++) {
            tmppnt[0] = i * node_dist + shift;
	    // tmppnt[1] = (1-umat->GetValue(j*xnum+i) - mindist)/(maxdist - mindist) * height;
	    tmppnt[1] = (1-values->GetValue(j*xnum+i)) *height;
	    heights[j][i]=tmppnt[1];	    
            tmppnt[2] = j * node_dist;
            pPoints->SetPoint(j*xnum+i,tmppnt);
          }
        }      
        break;

     default:	// grid off

        break;
  
   
   }
   vtkJPEGReader* reader3 = vtkJPEGReader::New();
   reader3-> SetFileName ("textures/boden.jpg");
   vtkTexture* atext1 = vtkTexture::New();
   atext1-> SetInput(reader3-> GetOutput());
   atext1-> InterpolateOn();

   
   cout<<" Texturen "<< endl;
 /*  std::vector <vtkTexture *> texturen;




   vtkJPEGReader* reader3 = vtkJPEGReader::New();
   reader3-> SetFileName ("textures/boden2_-3.jpg");
   vtkTexture* atext3 = vtkTexture::New();
   atext3-> SetInput(reader3-> GetOutput());
   atext3-> InterpolateOn();
   //  atext3  ->RepeatOff();
   texturen.push_back(atext3);


   vtkJPEGReader* reader4 = vtkJPEGReader::New();
   reader4-> SetFileName ("textures/boden2_-4.jpg");
   vtkTexture* atext4 = vtkTexture::New();
   atext4-> SetInput(reader4-> GetOutput());
   atext4-> InterpolateOn();
   //atext4 ->RepeatOff();
   texturen.push_back(atext4);

   
   vtkJPEGReader* reader5 = vtkJPEGReader::New();
   reader5-> SetFileName ("textures/boden2_-5.jpg");
   vtkTexture* atext5 = vtkTexture::New();
   atext5-> SetInput(reader5-> GetOutput());
   atext5-> InterpolateOn();
   // atext5 ->RepeatOff();
   texturen.push_back(atext5);
   
   vtkJPEGReader* reader7 = vtkJPEGReader::New();
   reader7-> SetFileName ("textures/boden2_-7.jpg");
   vtkTexture* atext7 = vtkTexture::New();
   atext7-> SetInput(reader7-> GetOutput());
   atext7-> InterpolateOn();
   // atext7 ->RepeatOff();
   texturen.push_back(atext7);

   vtkJPEGReader* reader8 = vtkJPEGReader::New();
   reader8-> SetFileName ("textures/boden2_-8.jpg");
   vtkTexture* atext8 = vtkTexture::New();
   atext8-> SetInput(reader8-> GetOutput());
   atext8-> InterpolateOn();
   // atext8 ->RepeatOff();
   texturen.push_back(atext8);

   vtkJPEGReader* reader9 = vtkJPEGReader::New();
   reader9-> SetFileName ("textures/boden2_-9.jpg");
   vtkTexture* atext9 = vtkTexture::New();
   atext9-> SetInput(reader9-> GetOutput());
   atext9-> InterpolateOn();
   // atext9 ->RepeatOff();
   texturen.push_back(atext9);

   vtkJPEGReader* reader11 = vtkJPEGReader::New();
   reader11-> SetFileName ("textures/boden2_-11.jpg");
   vtkTexture* atext11 = vtkTexture::New();
   atext11-> SetInput(reader11-> GetOutput());
   atext11-> InterpolateOn();
   //atext11 ->RepeatOff();
   texturen.push_back(atext11);

   vtkJPEGReader* reader12 = vtkJPEGReader::New();
   reader12-> SetFileName ("textures/boden2_-12.jpg");
   vtkTexture* atext12 = vtkTexture::New();
   atext12-> SetInput(reader12-> GetOutput());
   atext12-> InterpolateOn();
   // atext12 ->RepeatOff();
   texturen.push_back(atext12);


   vtkJPEGReader* reader13 = vtkJPEGReader::New();
   reader13-> SetFileName ("textures/boden2_-13.jpg");
   vtkTexture* atext13= vtkTexture::New();
   atext13-> SetInput(reader13-> GetOutput());
   atext13-> InterpolateOn();
   //atext13 ->RepeatOff();
   texturen.push_back(atext13);
  

   vtkJPEGReader* reader14 = vtkJPEGReader::New();
   reader14-> SetFileName ("textures/boden2_-14.jpg");
   vtkTexture* atext14 = vtkTexture::New();
   atext14-> SetInput(reader14-> GetOutput());
   atext14-> InterpolateOn();
   // atext14 ->RepeatOff();
   texturen.push_back(atext14);
   

   vtkJPEGReader* reader15 = vtkJPEGReader::New();
   reader15-> SetFileName ("textures/boden2_-15.jpg");
   vtkTexture* atext15 = vtkTexture::New();
   atext15-> SetInput(reader15-> GetOutput());
   atext15-> InterpolateOn();
   // atext15 ->RepeatOff();
   texturen.push_back(atext15);

 
 
   vtkJPEGReader* reader16 = vtkJPEGReader::New();
   reader16-> SetFileName ("textures/boden2_-16.jpg");
   vtkTexture* atext16 = vtkTexture::New();
   atext16-> SetInput(reader16-> GetOutput());
   atext16-> InterpolateOn();
   // atext16 ->RepeatOff();
   texturen.push_back(atext16);

   vtkJPEGReader* reader17 = vtkJPEGReader::New();
   reader17-> SetFileName ("textures/boden2_-17.jpg");
   vtkTexture* atext17 = vtkTexture::New();
   atext17-> SetInput(reader17-> GetOutput());
   atext17-> InterpolateOn();
   //atext17 ->RepeatOff();
   texturen.push_back(atext17);
  
  
   
   vtkJPEGReader* reader20 = vtkJPEGReader::New();
   reader20-> SetFileName ("textures/boden2_-20.jpg");
   vtkTexture* atext20 = vtkTexture::New();
   atext20-> SetInput(reader20-> GetOutput());
   atext20-> InterpolateOn();
   //atext20 ->RepeatOff();
   texturen.push_back(atext20);
  
   
   vtkJPEGReader* reader22 = vtkJPEGReader::New();  
   reader22-> SetFileName ("textures/boden2_-22.jpg");
   vtkTexture* atext22 = vtkTexture::New();
   atext22-> SetInput(reader22-> GetOutput());
   atext22-> InterpolateOn();
   //atext22 ->RepeatOff();
   texturen.push_back(atext22);
   

   
  cout<<"-------------- "<<endl;

   
    for( int j = 0; j < znum; j++ ) {
      for( int i = 0; i < xnum; i++) {
      //	int j=1 ;
      //int i=1;
     

	    vtkPoints* points = vtkPoints::New();
	    points->SetNumberOfPoints(4);
	    vtkPolyData *polyData = vtkPolyData::New();
	    polyData->Allocate(1,1);
	    vtkDataSetMapper* polyMap = vtkDataSetMapper::New();
	    vtkPolygon* quad = vtkPolygon::New();
	    vtkActor* gActor = vtkActor::New();	   
	    vtkFloatArray* pFloats = vtkFloatArray::New();
	    



	    double y1,y2,y3,y4;
	   
	    if (i>0&&j>0 && i<xnum-1 && j<znum-1){
	      
	      //links oben 
	       y1 = (heights[j-1][i-1]+heights[j][i-1]+heights[j-1][i]+heights[j][i])/4 ;	     	     
	      
	      //rechts oben
	       y4 = (heights[j-1][i+1]+heights[j-1][i]+heights[j][i+1]+heights[j][i])/4 ;
	     	    	    
	      //links unten
	       y2 =(heights[j+1][i-1]+heights[j][i-1]+heights[j+1][i]+heights[j][i])/4 ;
	      
	      //rechts unten
	       y3 = (heights[j+1][i+1]+heights[j][i+1]+heights[j+1][i]+heights[j][i])/4 ;	    	   
	     
	    }
	    else {
	    
	      // oben Kante
	      if (j==0 && i<xnum-1&& i>0){
		//links oben 
		 y1 = (heights[j][i-1]+heights[j][i])/2 ;
		      
		//rechts oben
		 y4 = (heights[j][i+1]+heights[j]cout <<cluster->GetRange()[0] << " ...... "<<cluster->GetRange()[1]<<endl;[i])/2 ;
		    	    
		//links unten
		 y2 =(heights[j+1][i-1]+heights[j][i-1]+heights[j+1][i]+heights[j][i])/4 ;
		   
		//rechts unten
		 y3 = (heights[j+1][i+1]+heights[j][i+1]+heights[j+1][i]+heights[j][i])/4 ;	

	      }	
	      
	       //linke Kante 
	      if (j<znum-1 && i==0&& j>0){
		//links oben 
		 y1 = (heights[j-1][i]+heights[j][i])/2 ;
		      
		//rechts oben
		 y4 = (heights[j-1][i+1]+heights[j-1][i]+heights[j][i+1]+heights[j][i])/4 ;	    
	    
		//links unten
		 y2 =(heights[j+1][i]+heights[j][i])/2 ;		   

		//rechts unten
		 y3 = (heights[j+1][i+1]+heights[j][i+1]+heights[j+1][i]+heights[j][i])/4 ;
		   

		}
	                //untere Kante
	      if (j==znum-1 && i<xnum-1 && i>0){
		//links oben 
		 y1 = (heights[j-1][i-1]+heights[j][i-1]+heights[j-1][i]+heights[j][i])/4 ;
		      
		//rechts oben
		 y4 = (heights[j-1][i+1]+heights[j-1][i]+heights[j][i+1]+heights[j][i])/4 ;	 
	    
		//links unten
		 y2 =(heights[j][i-1]+heights[j][i])/2 ;
	   
		//rechts unten
		 y3 = (heights[j][i+1]+heights[j][i])/2 ;
	
	   

		}
	      //rechte Kante
	      if (j<znum-1 && i==xnum-1 && j>0 ){
		//links oben 	    
		 y1 = (heights[j-1][i-1]+heights[j][i-1]+heights[j-1][i]+heights[j][i])/4 ;		
     
		//rechts oben
		 y4 = (heights[j-1][i]+heights[j][i])/2 ;
		    
		 //links unten
		 y2 =(heights[j+1][i-1]+heights[j][i-1]+heights[j+1][i]+heights[j][i])/4 ;
		       
		//rechts unten
		 y3 = (heights[j+1][i]+heights[j][i])/2 ;
	
	   
		}
	      //links oben Ecke 
	      if (j==0 && i==0){
		//links oben 
		 y1 =heights[j][i];
		 //rechts oben
		 y4 =(heights[j][i+1]+heights[j][i])/2 ;
		//links unten
		 y3 =(heights[j+1][i+1]+heights[j][i+1]+heights[j+1][i]+heights[j][i])/4 ;      
		 //rechts unten
		 y2 =(heights[j+1][i]+heights[j][i])/2 ;
	
	      }	
	      // rechts oben ecke
	      if (j==0 && i== xnum-1){
		//links oben 
		 y1 = (heights[j][i-1]+heights[j][i])/2 ;
		 //rechts oben	
		 y4 =heights[j][i];
		 //links unten	
		 y3 =(heights[j+1][i]+heights[j][i])/2 ;
		 //rechts unten    
		 y2 =(heights[j+1][i-1]+heights[j][i-1]+heights[j+1][i]+heights[j][i])/4 ;
	
	      }
	      //  links unten ecke 
	      if (j==znum-1 && i==0){
		//links oben 
		 y1 =(heights[j-1][i]+heights[j][i])/2 ;
		 //rechts oben			
		 y4 =(heights[j-1][i+1]+heights[j-1][i]+heights[j][i+1]+heights[j][i])/4 ;
		//links unten		
		 y3 =(heights[j][i+1]+heights[j][i])/2 ;
		 //rechts unten    
		 y2 =heights[j][i];
	
	      }
	      //rechts unten ecke
	      if (j==znum-1 && i==xnum-1){
		//links oben 
		y1 =(heights[j-1][i-1]+heights[j][i-1]+heights[j-1][i]+heights[j][i])/4 ;
		 //rechts oben
		y4 =(heights[j-1][i]+heights[j][i])/2 ;
		//links unten	
		y2 =(heights[j][i-1]+heights[j][i])/2 ;
		//rechts unten    
		y3 =heights[j][i];
	      }

	    }

	     
	      
	    //   points->InsertPoint(0, i*node_cout <<cluster->GetRange()[0] << " ...... "<<cluster->GetRange()[1]<<endl;dist - 0.5,y1,j*node_dist-0.5);
	    //   points->InsertPoint(1, i*node_dist - 0.5,y2,j*node_dist+0.5);	    	    	     
	    //  points->InsertPoint(3, i*node_dist + 0.5,y4,j*node_dist-0.5);	   
	    //  points->InsertPoint(2, i*node_dist + 0.5,y3,j*node_dist+0.5);
	   
	    //links oben  
	    points->InsertPoint(0, node_dist*(i - 0.5),y1, node_dist*(j-0.5) );	 
	    //links unten
	    points->InsertPoint(1, node_distrefVectors.items()*(i - 0.5),y2, node_dist*(j+0.5) );
	    //rechts oben
	    points->InsertPoint(3, node_dist*(i + 0.5),y4, node_dist*(j-0.5) );	
	    //rechts unten   
	    points->InsertPoint(2, node_dist*(i + 0.5),y3, node_dist*(j+0.5) );
	   
	   pFloats->InsertNextValue(y1/height);
	   pFloats->InsertNextValue(y2/height);
	   pFloats->InsertNextValue(y3/height);
	   pFloats->InsertNextValue(y4/height);

	    quad->GetPointIds()-> SetNumberOfIds (4);
	    quad->GetPointIds()-> SetId (0, 0);
	    quad->GetPointIds()-> SetId (1, 1);
	    quad->GetPointIds()-> SetId (2, 2);
	    quad->GetPointIds()-> SetId (3, 3);
	  
	    polyData->InsertNextCell(quad->GetCellType(),quad->GetPointIds());
	    polyData->SetPoints(points);
	    polyData->GetPointData()->SetScalars(pFloats);
	      
	    vtkPolyDataNormals* pNormals =  vtkPolyDataNormals::New();
	    pNormals->SetInput(polyData);
	    //   vtkTextureMapToCylinder * tMapPlane =vtkTextureMapToCylinder::New();
	    vtkTextureMapToSphere * tMapPlane = vtkTextureMapToSphere::New();
	    tMapPlane->SetInput(pNormals->GetOutput());
	    // vtkPlaneSource* plane = vtkPlaneSource::New();
     
	    polyMap->SetInput(tMapPlane->GetOutput());	
	    polyMap ->SetLookupTable( pLUT );  
	    gActor->SetMapper(polyMap);

	    

	    int choice =14-(int)(heights[j][i]/height*14);
	    //   cout << "choice: " <<choice<<endl;
	    //   cout<<"height "<< heights[j][i]/height<<endl;

	     gActor->SetTexture(texturen[choice]);
	    //  gActor->SetTexture(atext1);
	   
	    
	     //   gActor->RotateX(180);
	    
	    // gActor->GetProperty()->SetColor(1,0,0);
	    //   gridActorCollection->AddItem(gActor);
	      
	      }
	    }
   */

   pData->SetPoints(pPoints);
   

     cout << "now adding mesh" << endl;
     // add mesh
     int offs = 0;
     int iX, iZ;

     for( iZ=0; iZ < znum-1; iZ++ ) {
       for( iX=0; iX < xnum; iX++ ) {
         if( iX < xnum-1 ) {
           pCells->InsertNextCell(4);
           pCells->InsertCellPoint( iZ*xnum + iX );
           pCells->InsertCellPoint( iZ*xnum + iX + 1 );
           pCells->InsertCellPoint( (iZ+1)*xnum + iX );
           pCells->InsertCellPoint( (iZ+1)*xnum + iX + 1 );
           pData->SetStrips(pCells);
         }
         offs++;
       }
     }
     
      

     pData->GetPointData()->SetScalars( values );
     // pLUT->SetTableRange( mindist, maxdist );
     cout << "value range: " << pLUT->GetTableRange()[0] << " to " << pLUT->GetTableRange()[1] << endl;
     //  pLUT->SetHueRange( 0.52, 0.68);
     //  pLUT->SetValueRange(0.0f, 1.0f);
     //   pLUT->SetHueRange(0,1);
     //   pLUT->SetAlphaRange( 0.9, 1.0  );
     //  pLUT->SetValueRange(0.1f, 1.0f);
     //   pLUT->SetHueRange(0.0f, 1.0f);
     //  pLUT->SetSaturationRange(0.0f, 0.0f);
     // pLUT->SetSaturationRange(0.0f, 1.0f);

     /*   pLUT->SetNumberOfColors (256);
      pLUT->SetNumberOfTableValues(256);
      for (int i = 255; i >= 0; i--) {
	pLUT->SetTableValue(255-i,(float)i/255,(float)i/255,(float)i/255);
	}   */ 
     
     cout << "hue range: " << pLUT->GetHueRange()[0] << " to " << pLUT->GetHueRange()[1] << endl;
     pLUT->SetNumberOfTableValues(256*6);
    cout << " numbers of colors "<< pLUT->GetNumberOfTableValues()<<endl;

	vtkTriangleFilter * triangle = vtkTriangleFilter::New();
	triangle->SetInput(pData);
	triangle->GetOutput()->ReleaseDataFlagOn();
	vtkButterflySubdivisionFilter* butterfly = vtkButterflySubdivisionFilter::New();
	butterfly->SetInput(triangle->GetOutput());
	butterfly->SetNumberOfSubdivisions(1);
	butterfly->GetOutput()->ReleaseDataFlagOn();
	vtkPolyDataNormals* pNormals =  vtkPolyDataNormals::New();
	pNormals->SplittingOn();
        pNormals->SetFeatureAngle (70.0);
     	pNormals->SetInput(butterfly->GetOutput());
     	vtkTextureMapToPlane * tMapPlane = vtkTextureMapToPlane::New();
   
     tMapPlane->SetInput(pNormals->GetOutput());
     //    vtkPlaneSource* plane = vtkPlaneSource::New();
    /* vtkTriangleFilter * triangle = vtkTriangleFilter::New();
     triangle->SetInput(pData);
     vtkButterflySubdivisionFilter* butterfly = vtkButterflySubdivisionFilter::New();
     butterfly->SetInput(triangle->GetOutput());
     butterfly->SetNumberOfSubdivisions(0);
      //pmap->SetInput(pdata);
    */
     
     
     
     pMap->SetInput( tMapPlane->GetOutput());
     //  pMap->SetInput(plane->GetOutput());
     pMap->SetLookupTable( pLUT );
     gridActor->SetMapper(pMap);
     if (Settings::clustering==mode){
	gridActor->GetProperty()->SetInterpolationToPhong( ) ;
     }else {
     	gridActor->GetProperty()->SetInterpolationToPhong ( ) ;
     }
    //   gridActor->SetTexture(atext1);
     //  gridActor->SetPosition(30,30,0);
      gridActorCollection->AddItem(gridActor);

     
     return gridActorCollection;
   } 
 



void Grid::SetSOM( vtkFloatArray* som ) {
  this->som = som;
}

float** Grid::getHeights(){
  return heights;
}

void Grid::setCluster(vtkFloatArray* c) {

cout <<"setCluster --------------------------------------- "<<endl;
	int numberOfClusters = 0;
	for (int i = 0; i< c->GetNumberOfTuples(); i++){
		if(c->GetValue(i) >numberOfClusters) numberOfClusters = static_cast<int>(c->GetValue(i));
	}
	numberOfClusters++;
	vector <vector <int> > clusters (numberOfClusters);
	for (int i = 0; i< c->GetNumberOfTuples(); i++){
		int id = static_cast<int> (c->GetValue(i));
		clusters[id].push_back(i);
		//cout << id << "  "<<i<<endl;
	}
cout <<"gathering clusters done   "<<endl;
	vtkFloatArray * meansOfClusters =  vtkFloatArray::New();
	meansOfClusters->SetNumberOfComponents( this->som->GetNumberOfComponents());
	meansOfClusters->SetNumberOfTuples(numberOfClusters);
	for (int j = 0; j<numberOfClusters;j++){
		for (int l = 0; l<meansOfClusters->GetNumberOfComponents();l++){
			meansOfClusters-> SetComponent (j,l,0.0);
		}
	}

	for (int j = 0; j<numberOfClusters;j++){
		for(int k = 0; k<clusters[j].size();k++){
			for (int l = 0; l<meansOfClusters->GetNumberOfComponents();l++){
				int refID = clusters[j][k];
				float tmp = som->GetComponent(refID,l) + meansOfClusters->GetComponent(j,l);
				meansOfClusters-> SetComponent (j,l,tmp);
			}
		}
		//cout<<clusters[j].size()<<" ";
		for (int l = 0; l<meansOfClusters->GetNumberOfComponents();l++){
			float tmp =  meansOfClusters->GetComponent(j,l)/clusters[j].size();
			//cout <<meansOfClusters->GetComponent(j,l)<<" "<< tmp<<"T ";
			meansOfClusters-> SetComponent (j,l,tmp);
		}
	}

	vtkFloatArray * somMeansCluster =  vtkFloatArray::New();
	somMeansCluster->SetNumberOfComponents( this->som->GetNumberOfComponents());
	somMeansCluster->SetNumberOfTuples(this->som->GetNumberOfTuples());

	int count = 0;
	for (int j = 0; j<numberOfClusters;j++){
		for(int k = 0; k<clusters[j].size();k++){
			int refID = clusters[j][k];
			for (int l = 0; l<somMeansCluster->GetNumberOfComponents();l++){
                                int clusterID = static_cast<int>(c->GetValue(refID));
				float tmp =  meansOfClusters->GetComponent(clusterID,l);
				somMeansCluster->SetComponent (refID,l,tmp);
			}
		}
	}
cout <<"umat  "<<endl;
	cluster = Umat(somMeansCluster);
	somMeansCluster->Delete();
	meansOfClusters->Delete();
	

}

vtkFloatArray* Grid::Umat(vtkFloatArray * som) {
  int dim = -1;
  float acdist = 0.0;
  float tmpdist = 0.0;
  int neighnum = 0;
  maxdist = 0.0;
  mindist = 100000;
  vtkFloatArray * umatUnnormed = vtkFloatArray::New();
  umatUnnormed->SetNumberOfValues( xnum*znum);
  vtkFloatArray * umat = vtkFloatArray::New();
  umat->SetNumberOfValues( xnum*znum );

  if( som ) {

    dim = som->GetNumberOfComponents();
    cout << som->GetNumberOfTuples() <<endl;
        cout << xnum*znum <<endl;
    if( som->GetNumberOfTuples() == xnum*znum ) {
      for( int j = 0; j < znum; j++ ) {
        for( int i = 0; i < xnum; i++ ) {
	   // cout << i << ", " << j << ":" << endl;
           acdist = 0.0;
           neighnum = 0;
           // determine distance to all 4 neighbour nodes
           if( i-1 >= 0 ) {
              neighnum++;
              tmpdist = 0.0;
              for( int k = 0; k < dim; k++ ) {
                tmpdist += powf( som->GetValue( (j*xnum + i)*dim + k ) -
                                 som->GetValue( (j*xnum + i -1)*dim + k ) , 2 );
              }
              acdist += sqrtf(tmpdist);
           }
           if( j-1 >= 0 ) {
              neighnum++;
              tmpdist = 0.0;
              for( int k = 0; k < dim; k++ ) {
                tmpdist += powf( som->GetValue( (j*xnum + i)*dim + k ) - 
                                 som->GetValue( ((j-1)*xnum + i)*dim + k ) , 2 );
              }
              acdist += sqrtf(tmpdist);
           }
           if( i+1 < xnum ) {
              neighnum++;
              tmpdist = 0.0;
              for( int k = 0; k < dim; k++ ) {
                tmpdist += powf( som->GetValue( (j*xnum + i)*dim + k ) - 
                                 som->GetValue( (j*xnum + i+1)*dim + k ) , 2 );
              }
              acdist += sqrtf(tmpdist);
           }
           if( j+1 < znum ) {
              neighnum++;
              tmpdist = 0.0;
              for( int k = 0; k < dim; k++ ) {
                tmpdist += powf( som->GetValue( (j*xnum + i)*dim + k ) - 
                                 som->GetValue( ((j+1)*xnum + i)*dim + k ) , 2 );
              }
              acdist += sqrtf(tmpdist);
           }
           
           acdist /= (float)neighnum;
	
           if(acdist > maxdist ) {
              maxdist = acdist;
           }
           if(acdist < mindist ) {
              mindist = acdist;
           }
	   umatUnnormed->SetValue( j*xnum + i, acdist );
	 //  cout << acdist<<endl;
	   //	   cout<< acdist <<endl;
	   	//   cout<< "umat vlaue "<<acdist<<endl;
        }
      }
      
    } // if( som->GetNumberOfTuples() == xnum*znum )
  } // if( som )

  Normalizer* norm =new Normalizer() ;
  norm->normAll(umatUnnormed,umat,0,0);

  delete norm;
  umatUnnormed->Delete();

  return umat;
}


void Grid::SetHueRangeMin( int min ) {
   this->hue_min = ((float)min)/100;
   
   pLUT->SetHueRange(hue_min, hue_max);

   cout << hue_min << endl;
}

void Grid::SetHueRangeMax( int max ) {
   this->hue_max = ((float)max)/100;
   
   pLUT->SetHueRange(hue_min, hue_max);

   cout << hue_max << endl;
}

