#ifndef	__Grid
#define	__Grid

#include <qobject.h>

#include <vtkLookupTable.h>
#include <vtkPoints.h>
#include <vtkRenderer.h>
#include <vtkPolyDataMapper.h>
#include <vtkCellArray.h>

class Grid : public QObject {
Q_OBJECT

  public:
  // methods
  
  /// constructor - initialize data structures
  Grid();
  
  /// delete things
  virtual ~Grid();
  
  /// set dimension of som - has to fit number of tuples in the som float array
  void SetDim( int x, int z );
  
  /// set renderer
  /**
   * removes old actor from renderer, adds new actor
   */
 // void SetRenderer( vtkRenderer* ren );
  
  /// set mode of som grid
  /**
   * \param mode grid mode - 0: off, 1:component plane, 3: height determined by
   * unified distance matrix
   */
  void SetMode( int mode );

  /// set component for component plane
  /**
   *
   */
  void SetComponent( int component );

  /// set distance of som nodes from each other
  /**
   * this influences max/min height in random and unified distance matrix mode
   */
  void SetNodeDist( float dist );
  
  /// set height of glyphs over grid
  /** 
   * this means height over zero. So if the grid doesn't have nodes where 
   * y component is zero, the distance is smaller than this value.
   */ 
  void SetHeight( float height );
  
  /// set a som from which to take vectors for unified distance matrix mode
  void SetSOM( vtkFloatArray* som);

  void setCluster(vtkFloatArray* cluster);
  
  /// builds points, vtkPolyData and returns vktPolyDataMapper* for grid
  vtkActorCollection* Build();
  


  /// get the heights of grid nodes -> to match the heights of the fishes and the grid
  float** getHeights(); 

  public slots:
  
  void SetHueRangeMin( int min );
  void SetHueRangeMax( int max );
  

  private:
  // methods
  // compute Umatrix - distance to neighbors as value per node
  vtkFloatArray* Umat(vtkFloatArray* som);

  // compute Pmatrix -  density in orig space as value per node
  vtkFloatArray* Pmat();



  vtkFloatArray* cluster;

		
  // attributes
  int component	;	
  int mode;		
  float	node_dist;	///< distance of glyph centers from each other
  float height; 	///< height of glyphs over grid's zero
  int xnum;             
  int znum;		
  float maxdist;	///< value of maximal unified distance a node has
  float mindist;	///< value of minimal unified distance a node has
  float hue_min;
  float hue_max;
  // vtkFloatArray* umat;
  float** heights;
  vtkPoints* pPoints;
  vtkCellArray* pCells;
 // vtkFloatArray* pFloats;
  vtkPolyData* pData;
  vtkPolyDataMapper* pMap;
  vtkRenderer* pRender;
  vtkLookupTable* pLUT;
  vtkFloatArray* som;  
  vtkActorCollection* gridActorCollection;
		
};
#endif
