#ifndef __Renderer
#define	__Renderer

#include "GlyphWrapper.h"
#include "Settings.h"
#include "Grid.h"

#include <qwidget.h>
#include <qframe.h>

#include <vtkAssembly.h>
#include <vtkActorCollection.h>
#include <vtkRenderWindow.h>
#include <vtkRenderer.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkPicker.h>
#include <vtkInteractorStyleTerrain.h>

class vtkRenderer;
class vtkRenderWindow;
class vtkRenderWindowInteractor;
class vtkAssembly;




class Renderer : public QWidget {
Q_OBJECT

   public:
      // methods
      Renderer(QFrame* frame);
      ~Renderer();
      QWidget* GetWidget();
      
      // returns the pointer to renderer object
      static Renderer* GetMe();


      
      // creates a Renderer object and returns its pointer
      static Renderer* Create( QFrame* frame );
      
      // deletes renderer object
      static void Delete();
      
      /// returns the distance of glyph centers above SOM nodes
      static float GetDefaultHeight();
      
      /// sets distance of glyph centers above SOM nodes
      void SetDefaultHeight( int height );
      
      /// gets max radius of any cloud that is drawn
      float GetMaxRad();
      
      void Render();
      void Render( Settings* set );
      
      /// Thrown when values to render are out of bounds.
      /**
       * Values have to be in [0.0,1.0] to be mapped to features.
       * Values have to be in [-1.0,1.0] to resemble a position in 3d space.
       */
      class ValuesOutOfRangeException {};
      class PosIdxUsedForMapping {};
      
   public slots:
      Grid* GetGrid();
		
   private:
      // methods
      void init_single_glyphs(Settings* set);
      void init_multi_glyphs(Settings* set);
      void renderExternData(Settings* set);
      void set_som_positions(Settings* set);
      void display_grid(Settings* set);
		
      // attributes
      QWidget* vtkwidget;
      vtkRenderWindow* renwin;
      vtkRenderer* ren,*renOverview;
      vtkRenderWindowInteractor* iren;
      vtkPicker* ppick;
      GlyphWrapper** mainglyph;
      vtkActorCollection** glyphacts;
      Grid* grid;
      int glyphnum;
      int paramnum;
      float maxrad;
      
      GlyphWrapper::glyphType type;
//      vtkActor* gridact;

      static Renderer* me;
      static float default_height;
 public :
vtkRenderWindow*  GetRenderWindow (){
return renwin;
}
		
};
#endif


