#include "Settings.h"
#include "Fish.h"
#include "DSContainer.h"

#include <iostream>
#include <fstream>
#include <string>

using namespace std;


Settings::Settings() {

  this->somName = "no dataset";
  this->datasetName = "no dataset";
  this->mappingDatasetName = "no dataset";
  this->clusterName = "no dataset";

  this->vistype = none;
  this->gtype = GlyphWrapper::FISH;

  // since settings objects are initialized with the Fish glyph...
  this->mapdim = Fish::GetNumberOfParameters();
  this->mapping = init_array( this->mapdim, -1 );


  // We don't know dimensions of any data sets at this point.
//  this->grid = false;
  this->gridType = Settings::umat;
  this-> component = 0;
 
  
  this->height = 5.0;
  this->dist = -1;
  this->gdist = 15.0;
  
  this->hsi = true;
  this->radius = -1;
  this->minsize = 3.0f;
  this->maxsize = 4.0f;
  
  this->defaults = new float[Fish::GetNumberOfParameters()];
  defaults[0] = 0.5f;
  defaults[1] = 0.5f;
  defaults[2] = 0.5f;
  defaults[3] = 0.5f;
  defaults[4] = 0.5f;
  defaults[5] = 0.5f;

  defaults[6] = 0.5f;
  defaults[7] = 0.5f;
  defaults[8] = 0.5f;
  defaults[9] = 0.5f;
  defaults[10] = 0.5f;
  defaults[11] = 0.5f;
  
  defaults[12] = 0.9f;
  defaults[13] = 0.7f;
  defaults[14] = 0.2f;
  
  defaults[15] = 0.0f;
  defaults[16] = 0.0f;
  
  cout << "settings inited" << endl;
}


Settings::~Settings() {
   if(mapping) {
     cout << "deleting mapping (~Settings)" << endl;
     delete[] mapping;
   }

  

   cout << "deleting defaults (~Settings)" << endl;
   delete[] defaults;
}


void Settings::setVisType( int vistype ) {
   this->vistype = vistype;
}


int Settings::getVisType() {
   return this->vistype;
}

 void Settings::setSomDS( string name){
	this->somName = name;
}
    
string Settings::getSomDS(){
	return this->somName;
}

void Settings::setDataset( string name){
	this->datasetName = name;
}

string Settings::getDataset(){
	return this->datasetName;
}

void Settings::setMappingDataset( string name){
this->mappingDatasetName = name;
}

string Settings::getMappingDataset(){
	return this->mappingDatasetName;
}
void Settings::setClusterDS(string c){
	cout <<c<<endl;
	this->clusterName = c;
}
string Settings::getClusterDS(){
  return this->clusterName;
}

void Settings::setGlyphType( int type ) {
  int newdim = GlyphWrapper::GetNumberOfParameters( this->gtype );
  
  this->gtype = static_cast<GlyphWrapper::glyphType>(type);
  this->mapping = resize_array( this->mapping, this->mapdim, newdim, -1 );  
  this->mapdim = newdim;

}


int Settings::getGlyphType() {
  return this->gtype;
}


void Settings::setMapping( int* mapping ) {
//cout<<"setMapping(settings)"<<endl;
  for(int i = 0; i < this->mapdim; i++ ) {
     this->mapping[i] = mapping[i];
  }

}

void Settings::setMapping( int val, int idx) {
   if(idx >= this->mapdim) {
     cout << "idx out of range! - idx = " << idx;
     cout << "   mapping dim = " << this->mapdim << endl;
   } else {
     this->mapping[idx] = val;
   }
}


int* Settings::getMapping() {
  //cout<<"getMapping(settings)"<<endl;
//  this->mapping = init_array( this->mapping, this->mapdim, -1 );
  return this->mapping;
}

int Settings::getMapping( int idx ) {
   if(idx >= this->mapdim) {
     cout << "idx out of range! - idx = " << idx;
     cout << "   mapping dim = " << this->mapdim << endl;
     return -1;
   } else {
     return this->mapping[idx];
   }

   return -1; // this can't be reached.
}


int Settings::getMapDim() {
   return this->mapdim;
}





/*
void Settings::setGridEnabled( bool on ) {
  this->grid = on;
}

bool Settings::getGridEnabled() {
  return this->grid;
}
*/

void Settings::setGridType( int type ) {
  this->gridType = type;
}


int Settings::getGridType() {
  return this->gridType;
}

 void Settings::setComponent( int component ){
	this->component=component;
 }

 int Settings::getComponent( ){
   return this->component;
 }





void Settings::setHeightOverSOM( float height ) {
  this->height = height;
}


float Settings::getHeightOverSOM() {
  return this->height;
}


void Settings::setNodeDistance( float dist ) {
  this->dist = dist;
}


float Settings::getNodeDistance() {
  return this->dist;
}


void Settings::setHSI( bool on ) {
  this->hsi = on;
}


bool Settings::getHSI() {
  return this->hsi;
}


void Settings::setRadius( float radius ) {
  this->radius = radius;
}


float Settings::getRadius() {
  return this->radius;
}


void Settings::setGlyphSize( float min, float max ) {
  this->minsize = min;
  this->maxsize = max;
}


float Settings::getGlyphSizeMin() {
  return this->minsize;
}


float Settings::getGlyphSizeMax() {
  return this->maxsize;
}


void Settings::setGlyphDefaults( float* defaults ) {
  this->defaults = defaults;
}


float* Settings::getGlyphDefaults() {
  return this->defaults;
}


float Settings::getGlyphDefault( int idx ) {
  return defaults[idx];
}


void Settings::PrintSelf() {
   cout << endl;
   cout << "========================================================" << endl;
   cout << "data sets:         " << this->somName << " and " << this->datasetName << endl;
   cout << "glyph type:       " << this->getGlyphType() << endl;
   cout << "mapping:          ";
   for(int i = 0; i < 17; i++) {
     cout << this->getMapping( i ) << "  ";
   }

 
//   cout << "grid enabled:      " << this->getGridEnabled() << endl;
   cout << "grid type:         " << this->getGridType() << endl;
   cout << "height over som:   " << this->getHeightOverSOM() << endl;
   cout << "node distance:     " << this->getNodeDistance() << endl;
   cout << "hsi on?            " << this->getHSI() << endl;
   cout << "radius:            " << this->getRadius() << endl;
   cout << "glyph size:        min: " << this->getGlyphSizeMin() << "  max: " << this->getGlyphSizeMax() << endl;
   cout << "glyph defaults:    ";
   for(int i = 0; i < 17; i++) {
     cout << this->getGlyphDefault( i ) << "  ";
   }
   cout << endl;
   cout << "===================================" << endl;

}

void Settings::toFile(string fileName) {

   ofstream file(fileName.c_str());


   file << this->somName <<  endl;
   file << this->datasetName << endl;


   for(int i = 0; i < 17; i++) {
     file << this->mapping[i] << endl;
   }


//   cout << "grid enabled:      " << this->getGridEnabled() << endl;
   file << this->gridType << endl;
   file << this->height << endl;
   file << this->dist << endl;
   file << this->hsi << endl;
   file << this->radius << endl;
   file << this->minsize <<  endl;
   file << this->maxsize << endl;
   file.close();
}


void Settings::fromFile(string fileName) {

   ifstream file(fileName.c_str());
   string sBuf ;
   getline(file, sBuf,'\n');
   this->somName = sBuf ;
   getline(file, sBuf,'\n');
   this->datasetName = sBuf ;

   int iBuf;
   for(int i = 0; i < 17; i++) {
     file >> iBuf;
     this->mapping[i] = iBuf;
     cout << this->mapping[i] <<endl;
   }

   file >> iBuf;  this->gridType = iBuf;
   float fBuf;

   file >> fBuf;  this->height = fBuf;
   file >> fBuf;  this->dist = fBuf;
   file >> fBuf;  this->hsi = fBuf;
   file >> fBuf;  this->radius = fBuf;
   file >> fBuf;  this->minsize = fBuf;
   file >> fBuf;  this->maxsize = fBuf;
cout <<" LOADED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl;
PrintSelf();
}


int* Settings::init_array(int dim, int val) {
  int* array;

  cout << "array not allocated yet - initing" << endl;
  array = new int[dim];
  for(int i = 0; i < dim; i++) {
    array[i] = val;
  }
  
  return array;

}



int* Settings::init_array(int dim) {
  int* array;

  cout << "array not allocated yet - initing" << endl;
  array = new int[dim];
  for(int i = 0; i < dim; i++) {
    array[i] = i;
  }
  
  return array;

}


int*  Settings::resize_array( int* array, int olddim, int newdim ) {
   int* temparr;
   
   if( olddim != newdim ) {
     temparr = new int[olddim];
     for( int i = 0; i < olddim; i++ ) {
       temparr[i] = array[i];
     }
     delete[] array;

     array = new int[newdim];
     for( int i = 0; i < newdim; i++ ) {
        if( i < olddim ) {
           array[i] = temparr[i];
        } else {
           array[i] = i;
        }
     }
     delete[] temparr;
   }
   
   return array;
}



int* Settings::resize_array( int* array, int olddim, int newdim, int val ) {
   int* temparr;

   if( olddim != newdim ) {

     temparr = new int[olddim];
     for( int i = 0; i < olddim; i++ ) {
       temparr[i] = array[i];
     }
     delete[] array;

     array = new int[newdim];
     for( int i = 0; i < newdim; i++ ) {
        if( i < olddim ) {
           array[i] = temparr[i];
        } else {
           array[i] = val;
        }
     }
     delete[] temparr;

   }

   
   return array;
}





void Settings::setGridLeveling( float gdist ) {
   this->gdist = gdist;
}

float Settings::getGridLeveling() {
   return this->gdist;
}

void Settings::setDetail(int detail){
	this->detail=detail;
	Fish::SetLevelOfDetail(detail);	
}
int Settings::getDetail(){
	return this->detail;
}	
