#ifndef		__Settings
#define		__Settings

#include "GlyphWrapper.h"

#include <string>
using namespace std;

/// Class for storing data set specific settings. It can also write them to file.
class Settings {
  public:

    /// Constructor.
    Settings();

     /// Desctructor.
    ~Settings();

    void setVisType( int vistype );
    int getVisType();


    /// sets names of data sets.
    /**
     *Set the name of the som data ( the prototypvectors)
     *
     */
    void setSomDS( string name);
    
    /// \returns the som dataset name of this settings object
    string getSomDS();
    
    
    /**
     * Set the name of the dataset ( the vectors, from which the som was computed)
     *
     */
    void setDataset( string name);

    /// \returns the dataset name of this settings object
    string getDataset();

   /**
     * Set the name of the additional (mapping) dataset ( extra vectors which are used for mapping on the glyphs)
     *
     */
    void setMappingDataset( string name);
    
    /// \returns the additional (mapping) dataset name of this settings object
    string getMappingDataset();

     /**
     * Set the name of the cluster id dataset
     *
     */
    void setClusterDS( string name);

    /// \returns the dataset name of this settings object
    string getClusterDS();

    
    /// sets the glyph type to to use for visualization
    /** 
     * See GlyphWrapper.cc enum glyphType for a guess at which int number
     * is used for which glyph type.
     */
    void setGlyphType( int type = 0 );
    
    /// gets the glyph type to use for visualization
    int getGlyphType();
    
   

    /// sets the mapping of data set elements to glyph features
    void setMapping( int* mapping );
    
    /// sets single elements of the mapping of data set elements to glyph features
    /** 
     * See GlyphWrapper::SetMapping(int* mapping) for more detail.
     */
    void setMapping( int val, int idx );
    
    /// gets mapping array for data set elements to glyph features
    int* getMapping();
    
    /// gets single element of mapping array for data set elements to glyph features
    int getMapping( int idx );
    
    /// gets size of mapping array
    int getMapDim();
    
    /// sets grid type. 0 = none, 1 = plain, 2 = random, 3 = unified distance matrix
    void setGridType( int type );
    
    /// gets grid type
    int getGridType();
    
 /// set component for component plane

    void setComponent( int component );

    int getComponent( );

    /// sets the height of glyphs above the SOM grid.
    void setHeightOverSOM( float height );
    
    /// gets the height of glyphs above the SOM grid.
    float getHeightOverSOM();
    
    /// sets the distance of SOM node glyphs 
    void setNodeDistance( float dist );
    
    /// gets the distance of SOM node glyphs
    float getNodeDistance();
    
    /// sets the distance between the lowest and highest node in the SOM's grid
    /**
     * This is used for random and unified distance matrix grid modes only.
     * Only the y axes is considered.
     */
    void setGridLeveling( float gdist );
    
    /// gets the distance between the lowest and highest node in the SOM's grid
    float getGridLeveling();
    
    /// sets wether HSI or RGB color model is to be used. true for HSI.
    void setHSI( bool on );
    
    /// gets wether HSI or RGB color model is to be used. true for HSI.
    bool getHSI();
    
    void setRadius( float radius = -1 );
    float getRadius();
    
    /// sets the min and max glyph size for the current glyph.
    void setGlyphSize( float min, float max );
    
    /// gets the minimum glyph size for the current glyph.
    float getGlyphSizeMin();
    
    /// gets the maximum glyph size for the current glyph.
    float getGlyphSizeMax();
    
    /// sets an array of default values which are applied to glyph features which have not a value mapped onto them
    /** 
     * The array should have the size of the number of features the current
     * glyph has.
     */
    void setGlyphDefaults( float* defaults );
    
    // gets array of default values to apply to glyph features which havenot a value mapped onto them
    float* getGlyphDefaults();

    // gets single default value to apply to glyph features which havenot a value mapped onto them
    float getGlyphDefault( int idx );
    
    void setDetail(int detail);
    
    int getDetail();
    
    /// prints this settings object to stdout
    void PrintSelf();

    /// save this settings object
    void toFile(string file);

    /// load settings object
    void fromFile(string file);


   /* 1: only the prototypevectors, 2: prototypevectors with data (same number of components) 3: prototypevectors
	with data, but additional external data is mapped on the data glyphs, no proto glyphs
   */
  enum vistype { proto, protoWithData, externData, none };
  enum gridtype { umat ,componentPlane, clustering};
    
  private:
    int* init_array( int dim );
    int* init_array( int dim, int val );
    int* resize_array( int* array, int olddim, int newdim );
    int* resize_array( int* array, int olddim, int newdim, int val );
  
    int vistype;		///< visualization type - determines type that datasets are to have
    string somName ;
    string datasetName ;
    string mappingDatasetName;
    string clusterName;
    GlyphWrapper::glyphType gtype;

    int* mapping;		
    int mapdim;


    int component;
    int gridType;
    int detail;
    float height;
    float dist;
    float gdist;		///< leveling (distance between min/max height) of som
    bool hsi;
    float radius;
    float minsize, maxsize;
    float* defaults;
        
};
#endif
