#include "Gui.h"
#include "DataReader.h"
#include "DSContainer.h"
#include "Normalizer.h"
#include "GuiDataTables.h"
#include "GuiNormDialog.h"
#include <iostream>
#include <qapplication.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qframe.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qtabwidget.h>
#include <qinputdialog.h>

#include <qpushbutton.h>


using namespace std;


Gui::Gui (QWidget* parent , const char* name )  : QVBox( parent, name ){

   // has to be done first because this->setup depends on initialized renderer
   // build frame around vtk widget
   
   //QWidget dataWidget;  
   dataFrame = new QFrame( this, "Data" );
   dataFrame->setFrameStyle( QFrame::Sunken | QFrame::Panel );
   dataFrame->setLineWidth( 2 );

  
   visFrame = new QFrame( this, "Vis" );
   visFrame->setFrameStyle( QFrame::Sunken | QFrame::Panel );
   visFrame->setLineWidth( 2 );
 
   
   // create the renderer
   ren = Renderer::Create( visFrame );

   // build vtk widget
   cout << "begin to create renderer" << endl;
   vtkwidget = ren->GetWidget(); 
   vtklayout = new QHBoxLayout( visFrame, 2, 2,"vtk layout" );
   vtklayout->addWidget( vtkwidget, 1 );
   cout << "render window done" << endl;
   
  
   // initiate private vars
   this->tables   = new GuiDataTables();
   this->setup    = new GuiVisSetup();
//   this->settings = new GuiSettings();

   // connect signals to slots:
   connect(this , SIGNAL(DataSetAdded(QString)), tables, SLOT(AddDS(QString)) );
   connect(this, SIGNAL(DataSetAdded()), setup, SLOT(UpdateWidgets()) );
   // set properties for main application window
  // this->setCaption( "GlyphView 0.1" );


   // build file menu
   file = new QPopupMenu(  this);
   file->insertItem( "Add data set...", this, SLOT(AddDS()), CTRL+Key_O );
  // file->insertItem( "Import ml Data...", this, SLOT(ImportDS()), CTRL+SHIFT+Key_O );
   file->insertSeparator();
   file->insertItem( "Save selected data set...",this, SLOT(SaveDS()), CTRL+Key_S );
  // file->insertItem( "Save data set list...", this, SLOT(Nothing()), CTRL+SHIFT+Key_S );
  // file->insertItem( "Load data set list...", this, SLOT(Nothing()), CTRL+Key_L );
   file->insertSeparator();
   file->insertItem( "Quit", qApp, SLOT(quit()), CTRL+Key_Q );

   // build data set menu
   ds = new QPopupMenu( this);
   ds->insertItem( "Remove data set from list", this, SLOT(RemoveDS()), CTRL+Key_D );
 //  ds->insertItem( "Rename...", this, SLOT(Nothing()), CTRL+Key_R );
  // ds->insertItem( "Create Subset...", this, SLOT(Nothing()), CTRL+Key_C );
   ds->insertItem( "Normalize...", this, SLOT(NormalizeDS()), CTRL+Key_N );
  // ds->insertItem( "Show statistics...", this, SLOT(Nothing()), CTRL+Key_T );

   // build options menu
//   opt = new QPopupMenu( this );
//   opt->insertItem( "Set minimal size of vtk window...", this, SLOT(Nothing()), CTRL+SHIFT+Key_M );
	cout << "build menu done" << endl;

   // build menu bar
   menu = new QMenuBar(  this);
   menu->setSeparator( QMenuBar::InWindowsStyle );
   menu->insertItem( "&File", file );
   menu->insertItem( "&DS", ds );
//   menu->insertItem( "&Options", opt );
   cout << "menus done" << endl;   

   
   // build button widget (below vtk widget)
/*   buttons = new QHBoxLayout( 2,"buttons layout" );
   but1 = new QPushButton( "Show stats", this );
   but2 = new QPushButton( "Hide group", this );
   but3 = new QPushButton( "Show in data set table", this );
   buttons->addWidget( but1 );
   buttons->addWidget( but2 );
   buttons->addWidget( but3 );
*/

   // build vboxlayout (for vtk and info widget)
// left = new QVBoxLayout( 2, "left layout" ); 
 //left->addWidget( this->tables, 1 );
 //left->addLayout( buttons );


   // build tab widget
   tabs = new QTabWidget( this, "tabs");
   tabs->addTab( tables, "Data sets" );
   tabs->addTab( setup, "Mapping" );

   connect(setup, SIGNAL(switchToViz()), this, SLOT(toViz()) );
  // setup->show();
   
   tabs->addTab(visFrame, "Visualization");
   //cout << "tabs done" << endl;   
  //start = new QPushButton("Start",this);
  // connect( start, SIGNAL(clicked()), this, SLOT(Start()) );

   // build vboxlayout for tab widget:
   right = new QVBoxLayout( 2, "right layout" );
   right->addWidget(tabs);


   // build hboxlayout (for vboxlayout and tab widget)
   central = new QHBoxLayout( 2, "central" );
   //central->addLayout( left , 1 );
   central->addLayout( right );


   //build vboxlayout (for menu, central, and status bar)
   all = new QVBoxLayout( this, 2, 2, "all" );
   //all->addWidget( menu ); // menu bar
   all->addLayout( central); // central area

   cout << "everything done" << endl;
}


Gui::~Gui() {
  // delete singleton stuff
  Renderer::Delete();
  DSContainer::Delete();
  cout << "deleted Renderer and DSContainer singleton objects (~Gui)" << endl;
  
  // don't delete widgets, Gui is a widget itself and will dispose of them on its own.
}

void Gui::toViz(){

	tabs->setCurrentPage(2);
}

int Gui::AddDS() {
   QString dsname; 		// data set name
   QString openpath( "../data/" ); 	// TODO: replace this with path from user settings later
   QString filename;
   QString fopenerror = "File Error";
   DataReader dr;
   DSContainer* dscont = DSContainer::GetMe();

   QFileDialog fd(this, "gv file open dialog", TRUE);
   fd.setMode( QFileDialog::ExistingFile );
   fd.setDir( openpath );
   fd.setViewMode( QFileDialog::List );

   if ( fd.exec() == QDialog::Accepted ) {
      filename = fd.selectedFile();
      
      try {
         dr.Init();
         dr.OpenFile( filename.ascii() );
//         dr.GetDataArray()->PrintSelf( cout,3 );

         // check if array name already exists
         if( dscont->GetDS( dr.GetDataArray()->GetName() ) ) {
            bool nameok = false;
            while(!nameok) {
               bool ok;
               int idx = dscont->GetNumberOfArrays();
               QString name( dr.GetDataArray()->GetName() );
               
               QString index;
               index.setNum( idx );
               name.append( " " );
               name.append( index );
               QString text = QInputDialog::getText(
                                  "Name Collision", 
                                  "The data set you are loading has a name that already\n"
                                  "exists in your data set list or is empty."
                                  "Choose another name please:",
                                  QLineEdit::Normal, name, &ok, this );
               if( ok ) {
                  if(    strcmp(text.ascii(),dr.GetDataArray()->GetName()) 
                      && !text.isEmpty() 
                      && !dscont->GetDS( text.ascii() ) 
                    ) {
                     dr.GetDataArray()->SetName( text.ascii() );
                     nameok = true;
                  }
               } else {
                  return(1);  
               }
            } // while(!nameok)
         }
	 cout << " ////////"<<endl;
         dsname.setAscii( dr.GetDataArray()->GetName() );
         dscont->AddDS( dr.GetDataArray(), 
                        dr.GetDimensionX(), dr.GetDimensionY(),
                        dr.GetFormat() );

         emit DataSetAdded( dsname );
         emit DataSetAdded();
      }
      catch(DataReader::FileNotOpenedException fnoe) {
         QMessageBox::information( this, fopenerror,
            "File could not be opened for some reason.",
            QMessageBox::Ok );
      }
      catch(DataReader::WrongFormatException wfe) {
         QMessageBox::information(this, fopenerror, 
            "This file is not in \"GlyphView 0.1\" format.",
            QMessageBox::Ok);
      }
      catch(DataReader::UnknownDSTypeException udste) {
         QMessageBox::information(this, fopenerror,
            "Data set type is not known. Must be one of DATA, PROTO, or SOM.",
            QMessageBox::Ok);
      }
      catch(DataReader::NotEnoughFloatsException nefe) {
         QMessageBox::information(this, fopenerror,
            "Values given for dimension and number of data points\n"
            "is not consistent with number of float values that\n"
            "could be read from file.\nAborting.",
            QMessageBox::Ok);
      }
      catch(DSContainer::MaxDataSetNumberReachedException mdsnre) {
         QMessageBox::information(this, fopenerror, 
            "There are already 255 data sets in your data set list.\n"
            "Can not add more. Aborting.",
            QMessageBox::Ok);
      }
     /* catch(...) {
         QMessageBox::information(this, fopenerror,
            "Some strange error occured that the programmer of\n"
            "this application did not think of. Aborting.",
            QMessageBox::Ok);
      }*/

   }
   
   return(0);
}


void Gui::RemoveDS() {
   tables->RemoveDS();
   setup->UpdateWidgets();
}


void Gui::NormalizeDS() {
   QString dsname;
   dsname = tables->GetCurrentDS();
   DSContainer* cont = DSContainer::GetMe();


   if( !dsname.isEmpty() ) {
     vtkFloatArray* array = cont->GetDS( dsname.ascii() );
   
   
     if( array != NULL ) {
       QString str;

       int start, end, type;   
       int comp;
       float min, max;   
       
       GuiNormDialog gnd( array->GetNumberOfComponents(), this, "gnd" );
       if( gnd.exec() == QDialog::Accepted ) {
          vtkFloatArray* normArray = vtkFloatArray::New();
          start = gnd.GetStartIdx();
          end = gnd.GetEndIdx();
          type = gnd.GetNormalizationType();
          comp = gnd.GetComponent();
          min = gnd.GetMinVal();
          max = gnd.GetMaxVal();
          
          str.setAscii( array->GetName() );

          cout << "switching..." << endl;      
          switch( type ) {
             case 0:	// by tuple
                str.append( " normalized by tuple" );

                Normalizer::byTuple( array, normArray, start, end, min, max );
                break;
             
             case 1:	// one element/component
                str.append( " normalized by component" );
                Normalizer::oneElement( array, normArray, comp, min, max );
                break;

             case 2:	// by column
                str.append( " normalized by column" );
                Normalizer::allElements( array, normArray, start, end, min, max );
                break;
          
          
             case 3:	// all
                str.append( " normalized all" );
                Normalizer::normAll( array, normArray, start, end, min, max );
                break;
          
          }

          normArray->SetName( str.ascii() );
          normArray->SetNumberOfComponents( array->GetNumberOfComponents() );

          /*       
            cout << "array: tuples = " << array->GetNumberOfTuples() << "   dim = " << array->GetNumberOfComponents() << endl;
            cout << "normArray: tuples = " << normArray->GetNumberOfTuples() << "   dim = " << normArray->GetNumberOfComponents() << endl;
          */
          cont->AddDS( normArray, cont->GetSOMwidth( dsname.ascii() ), 
                                 cont->GetSOMheight( dsname.ascii() ),
                                 cont->GetDSType( dsname.ascii() )  );
         
          emit DataSetAdded( str );
          emit DataSetAdded();
          
          normArray->Delete();

       }
     } else {
           QString  error("No data set selected!");
           QMessageBox::information(this, error, 
              "Can't normalize since no data set is selected.",
              QMessageBox::Ok);
     
     }
   } else {
      QString error("There are no data sets!");
      QMessageBox::information(this, error,
          "Can't normalize data sets that don't exists...",
          QMessageBox::Ok);
   }
}

void Gui::Nothing() {
   cout << "Not implemented yet." << endl;
}


void Gui::Nothing(QString str) {
   cout << str.ascii() << endl;
}


void Gui::SaveDS() {
   QString savepath("../data/");
   QString filename = NULL;
   
   if(tables->GetCurrentDS()) {

      filename = QFileDialog::getSaveFileName(
                    savepath,
                    QString::null,
                    this,
                    "save file dialog",
                    "Choose a filename to save under" 
      );

      
      if(filename) {
         try{ 
            DataReader::SaveFile(filename.ascii(), tables->GetCurrentDS().ascii());
         }
         catch(DataReader::FileNotOpenedException fnoe) {
            QMessageBox::information( this, "Could not open file",
                                   "The filename you specified could not be written to.\n",
                                   QMessageBox::Ok 
            );
         }
      } 
   } else {
      QMessageBox::information( this, "Could not save",
                                "No data set is selected. Select a data set and try again,\n"
                                "please.\n",
                                QMessageBox::Ok
      );                                  
   }
   
}



int Gui::ImportDS() {
   QString dsname; 		// data set name
   QString openpath( "../data/" ); 	// TODO: replace this with path from user settings later
   QString filename;
   QString fopenerror = "File Error";
   DataReader dr;
   DSContainer* dscont = DSContainer::GetMe();

   QFileDialog fd(this, "ml file import dialog", TRUE);
   fd.setMode( QFileDialog::ExistingFile );
   fd.setDir( openpath );
   fd.setViewMode( QFileDialog::List );

   if ( fd.exec() == QDialog::Accepted ) {
     filename = fd.selectedFile();
      
     try {
        dr.Init();
        dr.ReadMLFile( filename.ascii() );

        dr.GetDataArray()->PrintSelf( cout,3 );

        bool nameok = false;
        while(!nameok) {
           bool ok;
           QString name( dr.GetDataArray()->GetName() );
           
           QString text = QInputDialog::getText(
                              "Name Collision", 
                              "Choose a name please:",
                              QLineEdit::Normal, name, &ok, this );
           if( ok ) {
              if(    strcmp(text.ascii(),dr.GetDataArray()->GetName()) 
                  && !text.isEmpty() 
                  && !dscont->GetDS( text.ascii() ) 
                ) {
                 dr.GetDataArray()->SetName( text.ascii() );
                 nameok = true;
              }
           } else {
              return(1);  
           }
        } // while(!nameok)

        dsname.setAscii( dr.GetDataArray()->GetName() );
        dscont->AddDS( dr.GetDataArray(), 
                        dr.GetDimensionX(), dr.GetDimensionY(),
                        dr.GetFormat() );

        emit DataSetAdded( dsname );
        emit DataSetAdded();
      }
      catch(DataReader::FileNotOpenedException fnoe) {
         QMessageBox::information( this, fopenerror,
            "File could not be opened for some reason.",
            QMessageBox::Ok );
      }
      catch(DataReader::WrongMLFormatException wfe) {
         QMessageBox::information(this, fopenerror, 
            "This file is not in \"CDataset 0.1\" format.",
            QMessageBox::Ok);
      }
      catch(...) {
         QMessageBox::information(this, fopenerror,
            "Some strange error occured that the programmer of\n"
            "this application did not think of. Aborting.",
            QMessageBox::Ok);
      }

   }
   
   return(0);
}


