#include "GuiDataTables.h"
#include "DSContainer.h"
#include "MyListViewItem.h"

#include <qstring.h>
#include <qlistview.h>
#include <qwidget.h>
#include <qlayout.h>
#include <vector>
#include <iostream>
using namespace std;

GuiDataTables::GuiDataTables(QWidget* parent, const char* name) : QWidget(parent,name) {

   // prepare data set list view
   table = new QListView( this,"data set list table" );
   table->setSelectionMode( QListView::Single );
   table->setSorting( true );

   table->addColumn( "ID" );
   table->addColumn( "Name" );
   table->addColumn( "Type" );
   table->addColumn( "#points" );
   table->addColumn( "Dimension" );
   table->addColumn( "SOM Dim X" );
   table->addColumn( "SOM Dim Y" );
   table->setSorting( 23 ); // any number larger than actual no of cols 
                          // let's the user decide wether and by what
                          // column to sort.
                          

   // prepare list view for contents of data set
   dataview = new QListView( this, "data set view list" );
   dataview->setSelectionMode( QListView::Single );
   dataview->setSorting( 10000 );


   // prepare layout
   lay = new QVBoxLayout(this, 0, 2, "table layout");
   lay->addWidget(table);
   lay->addWidget(dataview);

   connect( table, SIGNAL(selectionChanged(QListViewItem*)), 
            this,  SLOT(UpdateDSInfo(QListViewItem*)) );   
}

GuiDataTables::~GuiDataTables() {

}

void GuiDataTables::AddDS(QString dsname) {
   QString helpstr;
   string type;

   cout << dsname.ascii() << " added." << endl;   
   DSContainer* cont = DSContainer::GetMe();
   

   if(cont) {

     troot = new MyListViewItem(table);
     helpstr.setNum( cont->GetDSID(dsname.ascii()) );
     troot->setText( 0, helpstr.ascii() );
    
     troot->setText( 1, dsname.ascii() );

     type = cont->GetDSTypeAsString(dsname.ascii());
     troot->setText( 2, type.c_str() );

     helpstr.setNum( cont->GetDS(dsname.ascii())->GetNumberOfTuples() );
     troot->setText( 3, helpstr.ascii() );

     helpstr.setNum( cont->GetDS(dsname.ascii())->GetNumberOfComponents() );
     troot->setText( 4, helpstr.ascii() );

     helpstr.setNum( cont->GetSOMwidth(dsname.ascii()) );
     troot->setText( 5, helpstr );

     helpstr.setNum( cont->GetSOMheight(dsname.ascii()) );
     troot->setText( 6, helpstr );

     table->setColumnWidth( 1, table->columnWidth(1) + 5 );
     table->setColumnWidth( 2, table->columnWidth(2) + 5 );

   } else {

      cout << "cont is null" << endl;

   }
   
}


void GuiDataTables::RemoveDS() {

   QString dsname;
   QString tempstr;
   DSContainer* cont = DSContainer::GetMe();
   MyListViewItem* temp = dynamic_cast<MyListViewItem*>(table->currentItem());
   int pos;

  
   if(temp) {

      dsname.setAscii( temp->text(1) );
      table->takeItem( temp );
      delete temp;

      dataview->clear();
      while(dataview->columns()) {
         dataview->removeColumn(0);
      }

      cont->RemoveDS( dsname.ascii() );


      table->clear();
      for(int i = 0; i < cont->GetNumberOfArrays(); i++ ) {
         this->AddDS( cont->GetDSName(i) );
      }
     
   }


}



void GuiDataTables::UpdateDSInfo(QListViewItem* item) {
   
   QString dsname = item->text(1);
   QString str;
   vtkFloatArray* data = DSContainer::GetMe()->GetDS( dsname.ascii() );

   dataview->clear();
//   for(int i = 0; i < dataview->columns(); i++) {
   while(dataview->columns()) {
       dataview->removeColumn(0);   
   }


   dataview->addColumn("ID");
   for(int j = 0; j < data->GetNumberOfComponents(); j++) {
      str.setNum(j);
      dataview->addColumn(str);
   }
   

   for( int i = 0; i < data->GetNumberOfTuples(); i++) {
      MyListViewItem* temp = new MyListViewItem(dataview);
      str.setNum(i);
      temp->setText( 0, str );
      for(int j = 0; j < data->GetNumberOfComponents(); j++) {
         str.setNum( data->GetValue( i*data->GetNumberOfComponents() + j ) );
         temp->setText( j+1, str.ascii() );
      }
   }

   for( int i = 0; i < dataview->columns(); i++ ) {
      dataview->setColumnWidth( i, dataview->columnWidth(i) + 5 );
   }
///variance test
	int row = 0;

  vector <float> variancesSorted;
  vector <float> variancesUnSorted;
 for(int j = 0; j < data->GetNumberOfComponents(); j++) {
   	row=j;
   
	float mean = 0;
	float variance = 0;
	int number = data->GetNumberOfTuples();

	

	for (int i =0; i< number;i++){
		mean +=data->GetComponent(i,row);
	
	}
	mean/=number;
	for (int i =0; i< number;i++){
		variance +=(data->GetComponent(i,row)-mean) * (data->GetComponent(i,row)-mean);
	}
	variance/=(number-1);
	variancesSorted.push_back(variance);
	variancesUnSorted.push_back(variance);
	/*cout<<" row: "<< row<<endl;
	cout<<" mean: "<< mean<<endl;
        cout<<" variance: "<< variance<<endl;*/
 }
 sort(variancesSorted.begin(),variancesSorted.end());
 for(int i = 0; i < 17; i++) {
	for(int j = 0; j < data->GetNumberOfComponents(); j++) {
 		if (variancesSorted[data->GetNumberOfComponents()-i] == variancesUnSorted[j]){
			cout<<" row: "<< j<<endl;
	        	cout<<" variance: "<<variancesSorted[data->GetNumberOfComponents()-i]<<endl;
			j=data->GetNumberOfComponents();
		}
	}
 }

}



QString GuiDataTables::GetCurrentDS() {
   QString dsname;
   MyListViewItem* temp = dynamic_cast<MyListViewItem*>(table->currentItem());

   if(temp) {
      dsname.setAscii( temp->text(1) );
      return dsname;

   } else {
      return NULL;
   } // throw blah...
   
   
   return NULL;
}

