#include "MyListViewItem.h"
#include <qwidget.h>
#include <qstring.h>
#include <qlistview.h>
#include <qlayout.h>

#ifndef __GuiDataTables
#define __GuiDataTables


/// This class defines the layout of the part of the gui that shows data sets.
class GuiDataTables : public QWidget {
Q_OBJECT

  public:
    /// Constructor
    GuiDataTables(QWidget* parent = 0, const char* name = 0);
    
    /// Destructor
    ~GuiDataTables();
    
    /// \returns currently selected data set as a QString or QString::Empty if non is selected.
    QString GetCurrentDS();
    
  public slots:
    /// This lets new data sets appear in the data set list.
    /** 
     * used by Gui class when "Add data set..."  menu entry is invoked.
     * The addition of that data set from the internal data set registry
     * isn't done here, it's done in the Gui class.
     */
    void AddDS(QString);
    
    /// Removes currently selected data set from the visual data set list
    /** 
     * Used by Gui class when "Remove data set" menu entry is invoked.
     * The removal of that data set from the internal data set registry
     * isn't done here, it's done in the Gui class.
     */
    void RemoveDS();
    
    /// Updates display of data set contents when a data set is activated in the data set list.
    void UpdateDSInfo(QListViewItem*);
  
  private:
    QListView* table;
    QListView* dataview;
    QVBoxLayout* lay;
    MyListViewItem* troot;
    
};
#endif
