#include "GuiNormDialog.h"

#include <qwidget.h>
#include <qlayout.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qlineedit.h>

#include <iostream>
using namespace std;


GuiNormDialog::GuiNormDialog(int compnum, QWidget* parent, 
                     const char* name, bool modal) : QDialog(parent, name, modal) {

   QString lab("Choose normalization type and data columns to normalize");
   label = new QLabel( lab, this, "label" );
   
   QString normType1("Normalize each tuple");
   QString normType2("Normalize one column");
   QString normType3("Normalize each column");
   QString normType4("Normalize all");
   choose = new QComboBox( this, "chose" );
   choose->insertItem(normType1);
   choose->insertItem(normType2);
   choose->insertItem(normType3);
   choose->insertItem(normType4);

   QString startstr("first idx");
   start = new QLabel( startstr, this, "startlabel" );
   
   QString endstr("last idx");
   end = new QLabel( endstr, this, "endlabel" );
   
   startcombo = new QComboBox( this, "startcombo" );
   endcombo = new QComboBox( this, "endcombo" );
   
   QString compstr("component");
   component = new QLabel( compstr, this, "complabel" );
   compcombo = new QComboBox( this, "compcombo" );

   for( int i = 0; i < compnum; i++) {
      QString helpstr;
      helpstr.setNum( i );
      startcombo->insertItem( helpstr );
      endcombo->insertItem( helpstr );
      compcombo->insertItem( helpstr );
   }
   endcombo->setCurrentItem( compnum - 1 );
   component->hide();
   compcombo->hide();   

   QString minstr("lower bound");
   QString minval;
   minval.setNum( 0.0, 'g' );
   minlab = new QLabel( minstr, this, "minlabel" );
   minline = new QLineEdit( minval, dynamic_cast<QWidget*>(this), "minline" );
   mincheck = new QDoubleValidator( this, "mincheck" );
   minline->setValidator( mincheck );

   QString maxstr("upper bound");
   QString maxval;
   maxval.setNum( 1.0, 'g' );
   maxlab = new QLabel( maxstr, this, "maxlabel" );
   maxline = new QLineEdit( maxval, dynamic_cast<QWidget*>(this), "maxline" );
   maxcheck = new QDoubleValidator( this, "maxcheck" );
   maxline->setValidator( maxcheck );

   this->startint = startcombo->currentItem();
   this->endint   = endcombo->currentItem();
   this->type     = choose->currentItem();
   this->comp     = 0;
   this->min      = 0;
   this->max      = compnum;
   
   main = new QHBoxLayout( dynamic_cast<QWidget*>(this), 0, 3, "main layout of norm dialog" );
   left = new QVBoxLayout( main, 1, "left layout of norm dialog" ); 
   right = new QVBoxLayout( main, 1, "right layout of norm dialog" );
   
   ok = new QPushButton( "OK", dynamic_cast<QWidget*>(this) );
   connect( ok, SIGNAL(clicked()), this, SLOT(Done()) );
   
   cancel = new QPushButton( "Cancel", dynamic_cast<QWidget*>(this) );
   connect( cancel, SIGNAL(clicked()), this, SLOT(reject()) );
   
   left->addWidget( label );
   right->addWidget( choose );
   left->addWidget( start );
   right->addWidget( startcombo );
   left->addWidget( end );
   right->addWidget( endcombo );
   left->addWidget( component );
   right->addWidget( compcombo );
   left->addWidget( minlab );
   right->addWidget( minline );
   left->addWidget( maxlab );
   right->addWidget( maxline );

   left->addSpacing( 10 );
   right->addSpacing( 10 );
   left->addWidget( ok );
   right->addWidget( cancel );
   

   connect( choose, SIGNAL(activated(int)), this, SLOT(ChangedType(int)) );
   connect( startcombo, SIGNAL(activated(int)), this, SLOT(ChangedStart(int)) );
   connect( endcombo, SIGNAL(activated(int)), this, SLOT(ChangedEnd(int)) );
   connect( compcombo, SIGNAL(activated(int)), this, SLOT(ChangedComp(int)) );
   
}


GuiNormDialog::~GuiNormDialog() {

}


int GuiNormDialog::GetStartIdx() {
   return this->startint;
}

int GuiNormDialog::GetEndIdx() {
   return this->endint;
}

int GuiNormDialog::GetNormalizationType() {
   return this->type;
}

int GuiNormDialog::GetComponent() {
   return this->comp;
}

float GuiNormDialog::GetMinVal() {
   return this->min;
}

float GuiNormDialog::GetMaxVal() {
   return this->max;
}


void GuiNormDialog::ChangedType(int idx) {
   this->type = idx;
   
   start->hide();
   startcombo->hide();
   end->hide();
   endcombo->hide();
   component->hide();
   compcombo->hide();
   
   switch(idx) {
     case 0:	// by tuple
       start->show();
       startcombo->show();
       end->show();
       endcombo->show();
       break;
     
     case 1:	// normalize one element/component/column
       component->show();
       compcombo->show();
       break;
     
     case 2:	// by column
       start->show();
       startcombo->show();
       end->show();
       endcombo->show();
       break;


     case 3:	// all
       start->show();
       startcombo->show();
       end->show();
       endcombo->show();
       break;
       
     default:
       cerr << "idx of normalization type not as expected..." << endl;
       break;
   
   }
}

void GuiNormDialog::ChangedStart(int idx) {
   this->startint = idx;
}

void GuiNormDialog::ChangedEnd(int idx) {
   this->endint = idx;
}

void GuiNormDialog::Done() {
  this->min = minline->text().toFloat();
  this->max = maxline->text().toFloat();
  this->accept();
}

void GuiNormDialog::ChangedComp(int idx) {
   this->comp = idx;
}
