#include <qwidget.h>
#include <qstring.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qdialog.h>
#include <qvalidator.h>

#ifndef __GuiNormDialog
#define __GuiNormDialog

class GuiNormDialog : public QDialog {
Q_OBJECT

  public:
    /// Constructor
    /**
     * Constructs <i>paramnumber</i> Labels and Combo Boxes. Each QComboBox has 
     * <i>dim</i>+1 items with numbers 0 to <i>dim</i> and -1 to choose from.
     *
     * \param compnum Number of components the chosen dataset has.
     *
     * The user can choose a normalization type.
     * At the moment there are 3 types of normalization:
     * 0: by tuple - each tuple is normalized seperately.
     * 1: one element - only one column/component is normalized vertically.
     * 2: by column - each column is normalized seperately.
     * 3: all - all data within columns given by <i>startint</i> and 
     * <i>endint</i> is normalized.
     *
     * The user can choose a start index and end index for type
     * 0 and 2 which determine lower (<i>startint</i>) and upper 
     * (<i>endint</i>) bounds for which data columns are to be normalized.
     * If lower bound is higher than upper bound, no normalization is done.
     *
     * The user can choose a component to normalize if type 1 is selected.
     *
     * The user can choose a minimum and maximum value for normalization.
     * Note however that data can only be rendered if it's within [0,1].
     *
     */
    GuiNormDialog(int compnum, QWidget* parent = 0, const char* name = 0, bool modal = TRUE);
               
    /// Destructor
    ~GuiNormDialog();

    /// \returns index of lower bound for normalization.
    int GetStartIdx();

    /// \returns index of upper bound for normalization.
    int GetEndIdx();
    
    /// \returns type of normalization.
    /** 
     * At the moment there are 3 types of normalization:
     * 0: by tuple - each tuple is normalized seperately.
     * 1: one element - only one column/component is normalized vertically.
     * 2: by column - each column is normalized seperately.
     * 3: all - all data within columns given by <i>startint</i> and 
     * <i>endint</i> is normalized.
     */
    int GetNormalizationType();
    
    /// \returns component to normalize for type 1 (see above).
    int GetComponent();
    
    /// \return minimum. The lowest values in the data set are set to this value.
    float GetMinVal();
    
    /// \return maximum. The highest values in the data set are set to this value.
    float GetMaxVal();

  private slots:
     void ChangedType(int);
     void ChangedStart(int);
     void ChangedEnd(int);
     void ChangedComp(int);
     void Done();
    

  private:
   QLabel* label;
   QComboBox* choose;

   QLabel* start;
   QComboBox* startcombo;

   QLabel* end;
   QComboBox* endcombo;

   QLabel* component;
   QComboBox* compcombo;
   
   QLabel* minlab;
   QLineEdit* minline;
   QDoubleValidator* mincheck;
   
   QLabel* maxlab;
   QLineEdit* maxline;
   QDoubleValidator* maxcheck;
   
   
   QHBoxLayout* main;
   QVBoxLayout* left;
   QVBoxLayout* right;   
   
   QPushButton* ok;
   QPushButton* cancel;
   
   int startint;
   int endint;
   int type;      ///< type depends on normalization types currently available (there are four at the moment: 0: by tuple, 1: one component,2:by column, 3: all );
   int comp;
   float min;
   float max;

};
#endif
