#include "GuiVisSetup.h"
#include "GlyphWrapper.h"
#include "DataReader.h"
#include "DSContainer.h"
#include "Renderer.h"

#include <qwidget.h>
#include <qlayout.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qslider.h>
#include <vtkWindowToImageFilter.h>
#include <vtkBMPWriter.h>
#include <qstring.h>
#include <vtkRenderer.h>
#include <iostream>
#include <qfiledialog.h>
using namespace std;





void ComponentDialog::acceptInput(){

	this->set->setComponent(list->currentText().toInt());
	accept();
}

ComponentDialog::ComponentDialog(QWidget* parent, Settings* set)
	:QDialog(parent,"Dialog",true,0){


		this->set = set;
		DSContainer* cont = DSContainer::GetMe();
		int dim = cont->GetDS(set->getSomDS())-> GetNumberOfComponents ();
		QVBoxLayout* main = new QVBoxLayout(this,2,1,"main layout");
	  	list= new QListBox(this);
		main->addWidget(list);
		list->setMinimumSize(100,10*20);


		QString str;
		for (int i = 0; i <dim ; i++){
			list-> insertItem(str.setNum(i));
		}

		QHBox *buttonbox = new QHBox(this);
		main->addWidget(buttonbox);
		QPushButton * ok = new QPushButton("ok",buttonbox);
		QPushButton* abbr = new QPushButton("abbrechen",buttonbox);
		connect( ok, SIGNAL(clicked()),
			 this, SLOT(acceptInput()) );
		connect( abbr, SIGNAL(clicked()),
			this, SLOT(reject () ));
		}













int GuiVisSetup::visnum = 2;



GuiVisSetup::GuiVisSetup(QWidget* parent, const char* name) : QWidget(parent,name) {
  
   //Renderer* ren = Renderer::GetMe();
   

 
   QString vistype1("single glyphs on SOM nodes"); 
   QString vistype2("data above SOM");
   QString vistype3("data above SOM with extra Data for mapping");

   QString fish("Fish");
   QString info("Selected glyph type: ");

   glyphtype = new QComboBox(false, this, "glyph type combo box");
   glyphtype->insertItem( fish, GlyphWrapper::FISH );

   connect( glyphtype, SIGNAL(activated(int)), this, SLOT(gtypeChanged(int)) );

   vistype = new QComboBox(false, this, "vis type combo box");
   vistype->insertItem( vistype1 );
   vistype->insertItem( vistype2 );
   vistype->insertItem( vistype3 );
   vistype->setCurrentItem( 0 );
   
   connect( vistype, SIGNAL(activated(int)), this, SLOT(UpdateWidgets()) );
   cout << "combo boxes 1 done" << endl;   

   glyphlabel = new QLabel( this, "glyphlabel" );
   glyphlabel->setText( "Glyph Type:" );
   vistypelabel = new QLabel( this, "vistypelabel" );
   vistypelabel->setText( "Visualization Type:" );

   dslabel = new QLabel( this, "dslabel" );
   dslabel->setText( "Data set:" );
   dslabel->hide();
   somlabel = new QLabel( this, "somlabel" );
   somlabel->setText( "SOM prototype data set:" );
   somlabel->hide();
   cout << "labels done" << endl;


   /////////////////////////////////////////////////////////////////////
   // gui elements for choosing the data set
   dscombo = new QComboBox( false, this, "dscombo" );
   dscombo->hide();
   //dscombo->insertItem("no dataset");
   somcombo = new QComboBox( false, this, "somcombo" );
  // somcombo->insertItem("no dataset");
   somcombo->hide();

   
   extraDataLabel= new QLabel(this,"Extra Data");
   extraDataLabel->setText( "Extra Data used for Mapping:" );
   extraDataLabel->hide();
   extraDscombo = new QComboBox( false, this, "extraDscombo" );
   //extraDscombo ->insertItem("no dataset");
   extraDscombo->hide();

   clusterLabel = new QLabel(this,"clusterLabel");
   clusterLabel->setText( "Cluster Data used for Ground Coloring:" );
   clusterLabel->hide();
   clusterCombo = new QComboBox( false, this, "clusterCombo" );
  // clusterCombo->insertItem("no dataset");
   clusterCombo->hide();

   connect( somcombo, SIGNAL(activated(int)), this, SLOT(UpdateMapping()) );
   connect( dscombo, SIGNAL(activated(int)), this, SLOT(UpdateMapping()) );
   connect( extraDscombo, SIGNAL(activated(int)), this, SLOT(UpdateMapping()) );
   connect( clusterCombo, SIGNAL(activated(int)), this, SLOT(UpdateMapping()) );
   cout << "combo boxes 2 done" << endl;



   /////////////////////////////////////////////////////////////////////
   // main layout elements
   main = new QVBoxLayout(this, 1, 2, "main vis setup layout");
   header = new QHBoxLayout(main, -1, "header vis setup layout");
   titles = new QVBoxLayout(header, 2, "titles vis setup layout");
   combos = new QVBoxLayout(header, 2, "combos vis setup layout");
   cout << "layouts done" << endl;
   


   /////////////////////////////////////////////////////////////////////
   // gui elements for info widget (info on glyph type)
   infogroup = new QGroupBox( 2, Horizontal, this );
   info.append( fish );
  // infolabel = new QLabel( info, infogroup, "infolabel" );
   QString fname( GlyphWrapper::GetDescFile( glyphtype->currentItem() ).c_str() );
   fname.prepend( "img/");
   infoimg = new QPixmap( fname );
   pixlabel = new QLabel( infogroup, "pixlabel" );
   pixlabel->setPixmap( *infoimg );
  // pixlabel->setScaledContents(true);
   pixlabel->setMinimumSize(450,17*20);
      

   /////////////////////////////////////////////////////////////////////
   // gui elements for determining mapping of indices to features 
 // mappinggroup = new QGroupBox(1, Horizontal, "Choose mapping of data columns to features", 
       //                        this, "mappinggroup" );
	//mappinggroup = new QGroupBox(this);      
   int paramnum = GlyphWrapper::GetNumberOfParameters( GlyphWrapper::FISH );
   int dimension = 0;
   mappingwidget = new GuiMapping(paramnum, dimension, true, infogroup, "mapwidget");
   connect( mappingwidget, SIGNAL(MappingChangedSig(int)), this, SLOT(MappingChanged(int)) );
   //main->addWidget(mappinggroup);
   main->addWidget(infogroup);


   QButtonGroup* gridTypeGroup = new QButtonGroup(3, Qt::Horizontal,this,"gridtypeGroup");
   QRadioButton* gridTypeUmat = new QRadioButton(gridTypeGroup,"Umat");
   gridTypeUmat->setText("Umat");
  // gridTypeGroup->insert(gridTypeUmat);
   QRadioButton* gridTypeComponentPlane = new QRadioButton(gridTypeGroup,"ComponentPlane");
   gridTypeComponentPlane->setText("ComponentPlane");
   QRadioButton* gridTypeCluster = new QRadioButton(gridTypeGroup,"ClusterPlane");
   gridTypeCluster->setText("Cluster");
  // gridTypeGroup->insert(gridTypeComponentPlane);
   connect(gridTypeGroup, SIGNAL(clicked (int)), this, SLOT(gridTypeChanged(int)) );
   connect(gridTypeComponentPlane, SIGNAL(clicked ()), this, SLOT(chooseComponent()) );
   main->addWidget(gridTypeGroup);


   ///////////////////////////////////////////////////////////////////////////
   // gui elements for choosing height of glyphs over som, node distance, 
   // and leveling of the grid in case of random or unified dist matrix mode.
   addgroup = new QGroupBox( 1, Vertical, this );
   addframe = new QFrame( addgroup, "addframe" );
   addframe->setFrameStyle( QFrame::NoFrame );
   addlay = new QGridLayout( addframe, 4, 3, 0, -1, "addlay" );
   
   QString tempstr;
   tempstr.setNum(5.0);
   heightlab = new QLabel( "Height of nodes above SOM: ", addframe, "heightlab" );
   heightline = new QSlider( 5, 100, 1, 5, Qt::Horizontal, addframe, "heightline" );

  // doubvalh = new QDoubleValidator( 0.0, 100.0, 2, this, "doubvalh" );
  // heightline->setMaxLength( 5 );
  // heightline->setValidator( doubvalh );
   
   tempstr.setNum(-1);
   distlab = new QLabel( "Distance between nodes (leftmost for automatic): ", addframe, "distlab" );
   distline = new QSlider( -1, 100, 1, -1, Qt::Horizontal, addframe, "distline" );
		  
  // doubvald = new QDoubleValidator( -1.0, 100.0, 2, this, "doubvald" );
  // distline->setMaxLength( 5 );
  // distline->setValidator( doubvald );
   
   tempstr.setNum(5.0);
   umatlab = new QLabel( "Distance from highest to lowest grid point: ", addframe, "umatlab" );
   umatline = new QSlider( 5, 100, 1, 5, Qt::Horizontal, addframe, "umatline" );
  // umatline->setMaxLength( 5 );
   //umatline->setValidator( doubvalh );


   detail = new QSlider( 0, 4, 1, Fish::detail, Qt::Horizontal, addframe, "slidmin" );
  // slidmax = new QSlider( 0, 100, 1, 52, Qt::Horizontal, addframe, "slidmax" );
   connect( distline,SIGNAL(valueChanged(int)), this, SLOT(distChanged(int)) );
   connect( heightline,SIGNAL(valueChanged(int)), this, SLOT(heightChanged(int)) );
   connect( umatline,SIGNAL(valueChanged(int)), this, SLOT(gridheightChanged(int)) );

   connect( (QSlider*)detail, SIGNAL(valueChanged(int)), 
            this, SLOT(detailChanged(int)) );
  // connect( (QSlider*)slidmax, SIGNAL(valueChanged(int)), 
    //        this, SLOT(SetHueRangeMax(int)) );
cout << "connected" << endl;   
   
   QLabel* fakelab1 = new QLabel( "                    ", addframe, "fakelab1" );
   QLabel* fakelab2 = new QLabel( "                    ", addframe, "fakelab2" );
   QLabel* fakelab3 = new QLabel( "                    ", addframe, "fakelab2" );
   
   addlay->addWidget( heightlab, 1, 1 );
   addlay->addWidget( fakelab1, 1, 2 );
   addlay->addWidget( heightline, 1, 3 );

   addlay->addWidget( distlab, 2, 1 );
   addlay->addWidget( fakelab2, 2, 2 );
   addlay->addWidget( distline, 2, 3 );
   
   addlay->addWidget( umatlab, 3, 1 );
   addlay->addWidget( fakelab3, 3, 2 );
   addlay->addWidget( umatline, 3, 3 );
   
   QLabel* detaillab = new QLabel( "Level of Detail:", addframe, "detaillab" );
   
   cout << " adding widgets" << endl;
   addlay->addWidget( detaillab, 4, 1 );
   addlay->addWidget( detail, 4, 3 );
   
   addgroup->hide();
   main->addWidget( addgroup );


   
   cout << "done" << endl;


   /////////////////////////////////////////////////////////////////////
   // Visualization buttons...
   buttons = new QHBoxLayout( main, 4, "vis buttons" );
   QString savestr( "Save settings" );
   QString loadstr( "Load settings" );
   QString renderstr( "Render glyphs" );
   save = new QPushButton( savestr, this, "save button" );
   load = new QPushButton( loadstr, this, "load button" );
   render = new QPushButton( renderstr, this, "render button" );
   autoMapping = new QPushButton("Automatic Mapping" ,this , "automatic button" );
   render->setDisabled(true);
   connect( save, SIGNAL(clicked()), this, SLOT(SaveSettings()) );
   connect( load, SIGNAL(clicked()), this, SLOT(LoadSettings()) );
   connect( render, SIGNAL(clicked()), this, SLOT(Render()) );
    connect( autoMapping, SIGNAL(clicked()), this, SLOT(AutoMapping()) );

   buttons->addWidget( save );
   buttons->addWidget( load );

   buttons->addStretch( 1 );
   buttons->addWidget( autoMapping );
   buttons->addStretch( 1 );
   buttons->addWidget( render );


   /////////////////////////////////////////////////////////////////////
   // add everything to main layout elements   
   titles->addWidget(glyphlabel);
   titles->addWidget(vistypelabel);   
   titles->addWidget(dslabel);
   titles->addWidget(somlabel);
   titles->addWidget(extraDataLabel);
   titles->addWidget(clusterLabel);
   combos->addWidget(glyphtype);
   combos->addWidget(vistype);
   combos->addWidget(dscombo);
   combos->addWidget(somcombo);
   combos->addWidget(extraDscombo);
   combos->addWidget(clusterCombo);


   UpdateWidgets();   

   cout << "constructor done" << endl;
   
}


GuiVisSetup::~GuiVisSetup() {
   cout << "deleting infoimg (~GuiVisSetup)" << endl;
   delete infoimg;

   
   cout << "freeing set (~GuiVisSetup)" << endl;
  
}


void GuiVisSetup::Nothing(int i) {
   cout << "not implemented yet - int is " << i << endl;
}


int GuiVisSetup::getSettingsIndex(){
	
	
switch( vistype->currentItem() ) {
	case 0:		// prototypes on 3d positions
		if( somcombo->currentText() != 0) {
			for (unsigned int i =0; i< set.size();i++){
				if (set[i]->getSomDS() == somcombo->currentText() && set[i]->getVisType()==Settings::proto ){
					if (set[i]->getClusterDS() == clusterCombo->currentText() || (set[i]->getClusterDS()                                                      =="no dataset" && clusterCombo->currentText() ==0))
					return i;
				}
			}
		}
		break;

		case 1:		// single glyphs on som nodes
			if( somcombo->currentText() != 0  &&  dscombo->currentText() != 0 ) {
				for (unsigned int i =0; i< set.size();i++){
					if (set[i]->getSomDS() ==somcombo->currentText() && set[i]->getDataset()==dscombo->currentText() && set[i]->getVisType()==Settings::protoWithData ) {
						if (set[i]->getClusterDS() == clusterCombo->currentText() || (set[i]->getClusterDS()                                                      =="no dataset" && clusterCombo->currentText() ==0))
					return i;
					}
				}
			}
		break;

    		case 2:
			if( somcombo->currentText() != 0  &&  dscombo->currentText() != 0 && extraDscombo->currentText() != 0) {
				for (unsigned int i =0; i< set.size();i++){
					if (set[i]->getSomDS() ==somcombo->currentText() && set[i]->getDataset()==dscombo->currentText() &&set[i]->getMappingDataset() == extraDscombo->currentText() &&set[i]->getVisType()==Settings::externData  ) {
						 if (set[i]->getClusterDS() == clusterCombo->currentText() || (set[i]->getClusterDS()                                                     =="no dataset" && clusterCombo->currentText() ==0))
						return i;
						}
				}
			}
		break;
     
     

		default:
			break;
		}
	
	return -1;
}

void GuiVisSetup::UpdateWidgets() {
   QString helpstr;
   

   somcombo->clear();
   clusterCombo->clear();
   dscombo->clear();
   extraDscombo->clear();

   UpdateDataSets();

   dslabel->hide();
   dscombo->hide();
   somlabel->hide();
   somcombo->hide();
   extraDataLabel->hide();
   extraDscombo->hide();	
   clusterLabel->hide();
   clusterCombo->hide();
   //mappinggroup2->hide();
   //somgroup->hide();
   addgroup->hide();
   //protogroup->hide();
   
   switch(vistype->currentItem()) { // single glyphs on som nodes
     case 0:		
	     
	    
	     somlabel->show();
	     somcombo->show();
 	     clusterLabel->show();
             clusterCombo->show();

	     addgroup->show();
	     if( somcombo->count() != 0) {
		     render->setEnabled(true);
		    
	   
	     } else {
		     render->setEnabled(false);
	     }
        break;

	case 1:	// data above som
				
	     dslabel->show();
	     dscombo->show();
	     somlabel->show();
	     somcombo->show();
	     addgroup->show();
	     clusterLabel->show();
             clusterCombo->show();
	     if(dscombo->count() != 0 && somcombo->count() != 0) {
		     render->setEnabled(true);
		     

	     } else {
		     render->setEnabled(false);
	     }

        break;
     case 2:	// data above som

	     extraDataLabel->show();
	     extraDscombo->show();
	     dslabel->show();
	     dscombo->show();
	     clusterLabel->show();
             clusterCombo->show();
	     somlabel->show();
	     somcombo->show();
	     addgroup->show();
	     if(dscombo->count() != 0 && somcombo->count() != 0 && extraDscombo->count() != 0) {
		     render->setEnabled(true);
		    
	
	     } else {
		     render->setEnabled(false);
	     }

        break;
    


     default:
        cout << "this can't have happened!" << endl;
        
        break;

   }  

   UpdateMapping();

}



void GuiVisSetup::UpdateMapping() {
cout<<"--------UpdateMapping-------"<<endl;
  DSContainer* cont = DSContainer::GetMe();
  int dim  = 0;
  switch( vistype->currentItem() ) {
     case 0:		// prototypes on 3d positions
	     if( somcombo->currentText() != 0 ) {
		     QString temp = somcombo->currentText().ascii();
		     dim = cont->GetDS( temp )->GetNumberOfComponents();
		     int index =  getSettingsIndex();
 		     if (  index ==-1){

			set.push_back( new Settings() );
			set[set.size()-1]->setSomDS( somcombo->currentText().ascii() );
			if (clusterCombo->currentText()!=0){
				set[set.size()-1]->setClusterDS( clusterCombo->currentText().ascii() );
			}
			set[set.size()-1]->setVisType(Settings::proto);
			index = set.size()-1;
		     }
		     set[index]->PrintSelf();
		     mappingwidget->SetMapping( set[index]->getMapping() );
		     this->distline->setValue( static_cast <int>(set[index]->getNodeDistance()) );  
		     this->heightline->setValue( static_cast <int>(set[index]->getHeightOverSOM()) );
	     }
        break;

     case 1:		// single glyphs on som nodes
	     if( somcombo->currentText() != 0  &&  dscombo->currentText() != 0 ) {
		  
		     QString tempds = dscombo->currentText().ascii();
		     int index =  getSettingsIndex();
		     if (  index ==-1){
			set.push_back( new Settings() );
			set[set.size()-1]->setSomDS( somcombo->currentText().ascii() );
			set[set.size()-1]->setDataset( dscombo->currentText().ascii() );
			if (clusterCombo->currentText()!=0){
				set[set.size()-1]->setClusterDS( clusterCombo->currentText().ascii() );
			}
			set[set.size()-1]->setVisType(Settings::protoWithData);
			index = set.size()-1;
		     }
		     mappingwidget->SetMapping( set[index]->getMapping() );
		     this->distline->setValue( static_cast <int>(set[index]->getNodeDistance() ));
		     this->heightline->setValue( static_cast <int>(set[index]->getHeightOverSOM()) );
		     dim = cont->GetDS( tempds )->GetNumberOfComponents();

	     }
        break;
      case 2:	
     if( somcombo->currentText() != 0  &&  dscombo->currentText() != 0 &&extraDscombo->currentText() != 0) {
		     QString tempextraDs = extraDscombo->currentText().ascii();
		     int index =  getSettingsIndex();
		     if (  index ==-1){
			set.push_back(new Settings());
			set[set.size()-1]->setSomDS( somcombo->currentText().ascii() );
			set[set.size()-1]->setDataset( dscombo->currentText().ascii() );
			if (clusterCombo->currentText()!=0){
				set[set.size()-1]->setClusterDS( clusterCombo->currentText().ascii() );
			}
			set[set.size()-1]->setMappingDataset( extraDscombo->currentText().ascii() );
			set[set.size()-1]->setVisType(Settings::externData);
			index = set.size()-1;
		     }
		     mappingwidget->SetMapping( set[index]->getMapping() ); 
		     this->distline->setValue( static_cast <int>(set[index]->getNodeDistance() ));
		     this->heightline->setValue( static_cast <int>(set[index]->getHeightOverSOM()) );
		     cout<<tempextraDs<<endl;
		     dim = cont->GetDS( tempextraDs )->GetNumberOfComponents();

	     }
        break;

     default:
        break;
}
  cout << "in the middle of update..." << endl;

  mappingwidget->update(dim);
  //cout << "mappingwidget->paramnum = " << mappingwidget->GetParamnum() << endl;
  //cout << "settings for this vistype->paramnum = " << set[vistype->currentItem()]->getMapDim() << endl;



  //this->somgroup->setButton( set[vistype->currentItem()]->getGridType() );

  cout << "UpdateMapping done (GuiVisSetup)" << endl;
}


void GuiVisSetup::gtypeChanged(int idx) {
   /*int type = glyphtype->currentItem();

   // TODO! change info label according to glyph type
   
   // change mapping widget to reflect the number of parameters available
   mappingwidget->update( GlyphWrapper::GetNumberOfParameters( (GlyphWrapper::glyphType)type ), 
                          mappingwidget->GetDim() );
   mappingwidget->SetMapping( set[vistype->currentItem()]->getMapping() );
   for(int i = 0; i < visnum; i++) {
     set[i]->setGlyphType(type);
   }
   */
}


void GuiVisSetup::UpdateDataSets() {
   DSContainer* cont = DSContainer::GetMe();
   QString helpstr;

   for( int i = 0; i < cont->GetNumberOfArrays(); i++ ) {
   
      cont->PrintDS(i);
      switch( cont->GetDSType(i) ) {
         case DataReader::SOM:
            helpstr.setAscii(cont->GetDSName(i).c_str());
            somcombo->insertItem(helpstr);
            // for now, set last DS found to current data set. 
            // later this should be the preset for this vis type
            somcombo->setCurrentItem(somcombo->count()-1);
            
            break;
         
        /* case DataReader::PROTO:
            helpstr.setAscii(cont->GetDSName(i).c_str());
            protocombo->insertItem(helpstr);
            protocombo->setCurrentItem(protocombo->count()-1);
            break;
	*/
         case DataReader::DATA:
            helpstr.setAscii(cont->GetDSName(i).c_str());
            dscombo->insertItem(helpstr);
            dscombo->setCurrentItem(dscombo->count()-1);
 	    extraDscombo->insertItem(helpstr);
            extraDscombo->setCurrentItem(dscombo->count()-1);

            break;

	case DataReader::CLUSTER:
            helpstr.setAscii(cont->GetDSName(i).c_str());
            clusterCombo->insertItem(helpstr);
            clusterCombo->setCurrentItem(dscombo->count()-1);

            break;


         case DataReader::NONE:
            // nothing to do
            break;

         case DataReader::ERR:
            cout << "err!" << endl;
            break;

         default:
            cout << "type: " << cont->GetDSType(i) << endl;
            break;



      }
   }


}


void GuiVisSetup::SaveSettings() {

   QString openpath( "../settings/" ); 	// TODO: replace this with path from user settings later
   QString filename;
   QFileDialog fd(this, "Settings open dialog", TRUE);
   fd.setMode( QFileDialog::AnyFile );
   fd.setDir( openpath );
   fd.setViewMode( QFileDialog::List );

   if ( fd.exec() == QDialog::Accepted ) {
      filename = fd.selectedFile();
      this->set[getSettingsIndex()]->toFile(filename);
   }

}


void GuiVisSetup::LoadSettings() {

   QString openpath( "../settings/" ); 	// TODO: replace this with path from user settings later
   QString filename;
   QFileDialog fd(this, "Settings open dialog", TRUE);
   fd.setMode( QFileDialog::ExistingFile );
   fd.setDir( openpath );
   fd.setViewMode( QFileDialog::List );

   if ( fd.exec() == QDialog::Accepted ) {
      filename = fd.selectedFile();
      this->set[getSettingsIndex()]->fromFile(filename);
      }
   UpdateWidgets();   
}


void GuiVisSetup::EnableRenderButton(bool enabled) {
   this->render->setEnabled(enabled);
}

void GuiVisSetup::Render() {
   Renderer* ren = Renderer::GetMe();
   int paramnum = this->mappingwidget->GetParamnum();
  
  // int paramnum2 = this->mappingwidget2->GetParamnum();
  
   int curvistype;

   curvistype =  getSettingsIndex();
   try {   // because values could be out of range


	   cout<< "!!!!!!!!render!!!! " <<curvistype<<endl;



      // for mapping of indices to glyph features
      for( int i = 0; i < paramnum; i++ ) {
         set[curvistype]->setMapping( this->mappingwidget->GetCompForFeature(i), i );
      }

      // for mapping of data to som/prototypes
     // for( int i = 0; i < paramnum2; i++ ) {
      //   set[curvistype]->setProtoMapping( this->mappingwidget2->GetCompForFeature(i), i );
      //}
      
    // TODO: implement gui for grid and data setup
    //  set[curvistype]->setDataOn3DGrid( on/off );
    //  set[curvistype]->setDimsFor3DGrid( x, y, z );

    // TODO: implement gui for glyph defaults
      set[curvistype]->setHSI( false );
    //  set[curvistype]->setRadius( -1 fuer automatisch oder wert );
    //  set[curvistype]->setGlyphSize( min, max );
    //  set[curvistype]->setGlyphDefaults( array, so gross wie glyphtype features hat );
      cout << "ready to start Renderer" << endl;
      ren->Render( set[curvistype] );
      cout << "after Renderer" << endl;
      
      //QString multstr;
     // multstr.setNum( ren->GetMaxRad() );
     // multlab->setText( multstr );
      emit switchToViz();

   }    
   catch( Renderer::ValuesOutOfRangeException voore ) {
      QMessageBox::information( this, "Value(s) out of Range",
                    "One or more values of the data set(s) you specified are\n"
                    "out of range [0.0,1.0].",
                    QMessageBox::Ok
      );
   }
}

 void GuiVisSetup::AutoMapping(){

  DSContainer* cont = DSContainer::GetMe();
  string str ="no dataset";

  int curvistype;
  curvistype =  getSettingsIndex();



   switch( vistype->currentItem() ) {
     case 0:		// prototypes on 3d positions
	     if( somcombo->currentText() != 0 ) {
		str =  set[curvistype]->getSomDS();
	     }
        break;

     case 1:		// single glyphs on som nodes
	     if( somcombo->currentText() != 0  &&  dscombo->currentText() != 0 ) {
		str =  set[curvistype]->getDataset();
	     }
        break;
      case 2:	
      if( somcombo->currentText() != 0  &&  dscombo->currentText() != 0 &&extraDscombo->currentText() != 0) {
		str =  set[curvistype]->getMappingDataset();
	     }
        break;

     default:
        break;
   }

   if (str !="no dataset"){
	vtkFloatArray *data = cont->GetDS( str );




	 // first color, than length, last arc
	 int mappingFunction [17] = {10,8,7,5,3,1,11,9,6,4,2,0,16,15,14,12,13};



  	vector <pair <float,int> > variancesSorted;

 	for(int row = 0; row < data->GetNumberOfComponents(); row++) {


		float mean = 0;
		float variance = 0;
		int number = data->GetNumberOfTuples();

		for (int i =0; i< number;i++){
			mean +=data->GetComponent(i,row);
		}
		mean/=number;
		for (int i =0; i< number;i++){
			variance +=(data->GetComponent(i,row)-mean) * (data->GetComponent(i,row)-mean);
		}
		variance/=(number-1);
		variancesSorted.push_back(make_pair(variance,row));

		/*cout<<" row: "<< row<<endl;
		cout<<" mean: "<< mean<<endl;
		cout<<" variance: "<< variance<<endl;*/
 	}

 	sort(variancesSorted.begin(),variancesSorted.end());

	if (data->GetNumberOfComponents() >= 17){
		int count = 16;
		int diff =  (data->GetNumberOfComponents()-1)-16;
		for(int i = data->GetNumberOfComponents()-1; i >=diff ; i--) {
			cout<<" row: "<<variancesSorted[i].second <<endl;
			cout<<" variance: "<<variancesSorted[i].first<<endl;

			this->set[curvistype]->setMapping(variancesSorted[i].second, mappingFunction[count--]);

		}
	}else {
		int count = 16;
		for(int i = data->GetNumberOfComponents()-1; i >=0; i--) {
			cout<<" row: "<<variancesSorted[i].second <<endl;
			cout<<" variance: "<<variancesSorted[i].first<<endl;
			this->set[curvistype]->setMapping(variancesSorted[i].second, mappingFunction[count--]);
		}
	}
	mappingwidget->update(data->GetNumberOfComponents());
     }//if



 }

/*Settings* GuiVisSetup::GetSettings(int vistype) {
	QString tempproto = somcombo->currentText().ascii();
	QString tempds = dscombo->currentText().ascii();
	if (tempds == "")
		tempds="no dataset";
	int vistype =  getSettingsIndex(temp,tempds); 
   return this->set[vistype];
}*/



void GuiVisSetup::SetMapping(int val, int idx) {

	int vistype =  getSettingsIndex(); 
	if(vistype>-1)
  	this->set[vistype]->setMapping(val, idx);
}



void GuiVisSetup::MappingChanged(int which) {
	cout << "MappingChanged!" << endl;
   int paramnum = mappingwidget->GetParamnum();
   
   int vistype =  getSettingsIndex();
   
   if(vistype>-1)
   for( int i = 0; i < paramnum; i++ ) {
     this->set[vistype]->setMapping( mappingwidget->GetCompForFeature(i), i );
   }

   cout << "MappingChanged!" << endl;

}


/*void GuiVisSetup::ProtoMappingChanged(int which) {
   int paramnum = mappingwidget2->GetParamnum();
   
   for( int i = 0; i < paramnum; i++ ) {
     this->set[vistype->currentItem()]->setProtoMapping( mappingwidget2->GetCompForFeature(i), i );
   }

   cout << "ProtoMappingChanged!" << endl;
}


void GuiVisSetup::GridTypeChanged( int grid ) {
  
  cout << "gridtype: " << grid << endl;
  
  this->set[vistype->currentItem()]->setGridType( grid );

}*/

void GuiVisSetup::distChanged(int dist) {
	
	int vistype =  getSettingsIndex(); 
	if(vistype>-1)
   this->set[vistype]->setNodeDistance( dist );
   cout << "new dist: " << dist << endl;
}


void GuiVisSetup::heightChanged(int height) {
	
	int vistype =  getSettingsIndex(); 
	if(vistype>-1)
   this->set[vistype]->setHeightOverSOM( height );
   cout << "new height: " << height << endl;
}

/*void GuiVisSetup::protoDistChanged() {
   float dist = protoline->text().toFloat();

   this->set[vistype->currentItem()]->setProtoDist( dist );
   cout << "new proto dist: " << dist << endl;
}*/


void GuiVisSetup::gridheightChanged(int gdist) {

	int vistype =  getSettingsIndex(); 
	if(vistype>-1)
   this->set[vistype]->setGridLeveling(gdist);
   cout << "new grid leveling: " << gdist << endl;
}

void GuiVisSetup::detailChanged(int detail) {
	
	int vistype =  getSettingsIndex();
	if(vistype>-1)
		this->set[vistype]->setDetail(detail);
	cout << "new detail level: " << detail << endl;
}
void GuiVisSetup::gridTypeChanged(int gridType){
        int vistype =  getSettingsIndex();
	if(vistype>-1)
		this->set[vistype]->setGridType( gridType);
	cout << "new gridType: " << gridType << endl;
}
  void GuiVisSetup::chooseComponent(){
	 int vistype =  getSettingsIndex();
	 if(vistype>-1){
		ComponentDialog* d = new ComponentDialog(this,this->set[vistype]);
		d->show();
	 }


  }



