#ifndef vtkQGLRenderWindowInteractor_h
#define vtkQGLRenderWindowInteractor_h

//===========================================================
// now we define the C++ class

#include <qobject.h>
#include <qtimer.h>
#include <qmap.h>
#include "vtkVersion.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"

//class vtkQGLRenderWindow ;

// ------------------------------------------------------------
// :TRICKY: Fri Apr 21 22:19:27 2000 Pagey
// The Signal/Slot mechanism used by Qt requires that QObject 
// appear as the first class when using multiple inheritance. 
// Hence the order of the two classes QObject and vtkRenderWindowInteractor
// matters here. Be careful not to change it by accident. 
// ------------------------------------------------------------
class vtkQGLRenderWindowInteractor : 
  public QObject, public vtkRenderWindowInteractor
 {
  Q_OBJECT 
//#if QT_VERSION >= 200
//static void staticMetaObject1();
//#endif
public:
  static vtkQGLRenderWindowInteractor *New() ; 

  vtkTypeMacro(vtkQGLRenderWindowInteractor,vtkRenderWindowInteractor);

  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Initializes the event handlers without an XtAppContext.  This is
  // good for when you don't have a user interface, but you still
  // want to have mouse interaction.
  virtual void Initialize();

  // Description:
  // This will start up the X event loop and never return. If you
  // call this method it will loop processing X events until the
  // application is exited.
  virtual void Start();
  
  // Description:
  // Enable/Disable interactions.  By default interactors are enabled when
  // initialized.  Initialize() must be called prior to enabling/disabling
  // interaction. These methods are used when a window/widget is being
  // shared by multiple renderers and interactors.  This allows a "modal"
  // display where one interactor is active when its data is to be displayed
  // and all other interactors associated with the widget are disabled
  // when their data is not displayed.
  virtual void Enable();
  virtual void Disable();

  // Description:
  // Event loop notification member for Window size change
//  virtual void UpdateSize(int x,int y);

  // Description:
  // Timer methods must be overridden by platform dependent subclasses.
  // flag is passed to indicate if this is first timer set or an update
  // as Win32 uses repeating timers, whereas X uses One shot more timer
  // if flag==VTKXI_TIMER_FIRST Win32 and X should createtimer
  // otherwise Win32 should exit and X should perform AddTimeOut()
  virtual int CreateTimer(int ) ; 
  virtual int DestroyTimer() ; 

  // Description:
  // This function is called on 'q','e' keypress if exitmethod is not
  // specified and should be overidden by platform dependent subclasses
  // to provide a termination procedure if one is required.
  virtual void TerminateApp(void) { /* empty */ }

  // Description:
  // These methods correspond to the the Exit, User and Pick
  // callbacks. They allow for the Style to invoke them.
  //virtual void ExitCallback();
  //virtual void UserCallback();
  //virtual void StartPickCallback();
  //virtual void EndPickCallback();

protected:
  vtkQGLRenderWindowInteractor();
  ~vtkQGLRenderWindowInteractor();
  vtkQGLRenderWindowInteractor(const vtkQGLRenderWindowInteractor&) {};
  void operator=(const vtkQGLRenderWindowInteractor&) {};

  //
  // Timer used during various mouse events to figure 
  // out mouse movements. 
  //
  QTimer *mTimer ;

  QMap<int, const char*> mKeyMap;

  //
  // Location of the mouse as obtained from the latest
  // mouse event.
  //
  int mX, mY ;

  int MouseInWindow;

  //
  // User for switching to stereo mode.
  //
  int PositionBeforeStereo[2];
  //
  // Connect/disconnect all slots which allow events on the
  // render window to be passed to the interactor. 
  //
  void ConnectSlots() ;
  void DisconnectSlots() ;
  void ProcessSlotConnections(bool conn) ;

 public slots:
  virtual void MouseMove(const QMouseEvent *event) ;
  virtual void LeftButtonPressed(const QMouseEvent *event) ;
  virtual void LeftButtonReleased(const QMouseEvent *event) ;
  virtual void MiddleButtonPressed(const QMouseEvent *event) ;
  virtual void MiddleButtonReleased(const QMouseEvent *event) ;
  virtual void RightButtonPressed(const QMouseEvent *event) ;
  virtual void RightButtonReleased(const QMouseEvent *event) ;
  // virtual void ButtonPressed(const QMouseEvent *event) ;
  // virtual void ButtonReleased(const QMouseEvent *event) ;
  virtual void KeyPressed(QKeyEvent *event) ;
  virtual void KeyReleased(QKeyEvent *event);

 private slots:
  void TimerFunc() ;

/*
  signals:
  void RenderWindowModified() ;*/
};

#endif
