// vtkQGLWindowQT.cpp: Implementierung der Klasse vtkQGLWindowQT.
//
//////////////////////////////////////////////////////////////////////

#include "vtkQGLWindowQT.h"


#include "moc_vtkQGLWindowQT.cpp"


vtkQGLWindowQT *vtkQGLWindowQT::mCurrentWidget = NULL;


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////


vtkQGLWindowQT::vtkQGLWindowQT(const QGLFormat & format, QWidget *parent, const char *name, const QGLWidget *shared) : QGLWidget(format, parent, name, shared)
{
	this->setFocusPolicy(QWidget::StrongFocus);

	this->setAutoBufferSwap(true);

	QSizePolicy policy = this->sizePolicy();
	policy.setHorData(QSizePolicy::Expanding);
	policy.setVerData(QSizePolicy::Expanding);
	this->setSizePolicy(policy);
}


void vtkQGLWindowQT::Frame(void)
{
	if ((!this->autoBufferSwap() ) &&			// buffers are not switched automatically
	 	 this->doubleBuffer())    // double buffering is enabled on QGLWidget side
	{
		// VTK wants us to swap buffers
		this->swapBuffers();
	}
}


void vtkQGLWindowQT::SetSize(int w, int h)
{
	QSize size;
	size.setWidth(w);
	size.setHeight(h);
	this->setSizeHint(size);
	this->resize(w, h);
}



void vtkQGLWindowQT::mouseMoveEvent( QMouseEvent *event ) 
{
	emit MouseMove(event) ;
}


void vtkQGLWindowQT::mousePressEvent( QMouseEvent *event ) 
{
	//
	// Emit a ButtonPressed signal for all mouse presses.
	//
	// emit ButtonPressed(event) ;
	//
	// Emit a signal for the button which was pressed.
	//
	switch(event->button()) 
	{
	case LeftButton:
		emit LeftButtonPressed(event) ;
		break ;
	case MidButton:
		emit MiddleButtonPressed(event) ;
		break ;
	case RightButton:
		emit RightButtonPressed(event) ;
		break;
	default:
		break ;
	}
	return ;
}


void vtkQGLWindowQT::mouseReleaseEvent( QMouseEvent *event ) 
{
	//
	// Emit a ButtonPressed signal for all mouse releases.
	//
	// emit ButtonReleased(event) ;
	//
	// Emit a signal for the the mouse button which was
	// released.
	//
	switch(event->button()) 
	{
	case LeftButton:
		emit LeftButtonReleased(event) ;
		break ;
	case MidButton:
		emit MiddleButtonReleased(event) ;
		break ;
	case RightButton:
		emit RightButtonReleased(event) ;
		break;
	default:
		break ;
	}
	return ;
}


void vtkQGLWindowQT::keyPressEvent (QKeyEvent * event) 
{
	//
	// Let the interactor handle this.
	//
	emit KeyPressed(event) ;
}


void vtkQGLWindowQT::keyReleaseEvent (QKeyEvent * event) 
{
	//
	// Let the interactor handle this.
	//
	emit KeyReleased(event) ;
}


QSize vtkQGLWindowQT::sizeHint(void) const
{
	return this->renderWindowSizeHint;
}


void vtkQGLWindowQT::setSizeHint(const QSize size)
{
	this->renderWindowSizeHint=size;
	this->updateGeometry();
}
