// vtkQtObjectFactory.cpp: Implementierung der Klasse vtkQtObjectFactory.
//
//////////////////////////////////////////////////////////////////////

#include "vtkQtObjectFactory.h"
#include "vtkQGLRenderWindow.h"
#include "vtkQGLRenderWindowInteractor.h"
#include "vtkQGLLight.h"
#include "vtkQtOutputWindow.h"
#include "vtkQtOpenGLPolyDataMapper.h"
#include "vtkPolyDataNoDeepCopyProb.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////


int			vtkQtObjectFactory::registered			= 0;
QWidget*	vtkQtObjectFactory::widgetParent		= NULL;
char*		vtkQtObjectFactory::widgetName			= NULL;
int			vtkQtObjectFactory::mirrorOpenGL		= 0;
int			vtkQtObjectFactory::compileAndExecute	= 0;
QGLWidget*	vtkQtObjectFactory::widgetShared		= NULL;


vtkQtObjectFactory*
vtkQtObjectFactory::New()
{
	return new vtkQtObjectFactory;
};


void
vtkQtObjectFactory::SetQtBase(QWidget* widgetParent, char* widgetName, QGLWidget* widgetShared)
{
	vtkQtObjectFactory::widgetParent = widgetParent;
	vtkQtObjectFactory::widgetName	 = widgetName;
	vtkQtObjectFactory::widgetShared = widgetShared;
}



void
vtkQtObjectFactory::GetQtBase(QWidget* &widgetParent, char* &widgetName, QGLWidget* &widgetShared)
{
	widgetParent = vtkQtObjectFactory::widgetParent;
	widgetName   = vtkQtObjectFactory::widgetName;
	widgetShared = vtkQtObjectFactory::widgetShared;
	vtkQtObjectFactory::widgetParent = NULL;
	vtkQtObjectFactory::widgetName   = NULL;
	vtkQtObjectFactory::widgetShared = NULL;
}



QWidget*
vtkQtObjectFactory::QtWidget(vtkObject* window)
{
	if(!vtkQtObjectFactory::registered)
		return NULL;
	return (dynamic_cast<vtkQGLPtrToQT*>(window))->GetQTPtr();
}



vtkObject*
vtkQtObjectFactory::VtkWindow(QWidget* widget)
{
	if(!vtkQtObjectFactory::registered)
		return NULL;
	return (dynamic_cast<vtkQGLPtrToVTK*>(widget))->GetVTKPtr();
}



void
vtkQtObjectFactory::RegisterObjectFactory()
{
	if (vtkQtObjectFactory::registered == 0)
	{
		vtkObjectFactory *of = vtkQtObjectFactory::New();
		vtkObjectFactory::RegisterFactory(of);
		vtkQtObjectFactory::registered = 1;
	}
};



vtkObject*
vtkQtObjectFactory::CreateObject(const char* vtkclassname)
{
	if(strcmp(vtkclassname, "vtkRenderWindow") == 0)
	{
		vtkObject *o = vtkQGLRenderWindow::New();
		return o;
	}
	if(strcmp(vtkclassname, "vtkRenderWindowInteractor") == 0)
	{
		return vtkQGLRenderWindowInteractor::New();
	}
	if(strcmp(vtkclassname, "vtkLight") == 0)
	{
		return vtkQGLLight::New();
	}
	if(strcmp(vtkclassname, "vtkPolyData") == 0)
	{
		return vtkPolyDataNoDeepCopyProb::New();
	}
	if (this->GetCompileAndExecute() != 0)
	{
		// Use the one step CompileAndExecute
		if(strcmp(vtkclassname, "vtkOpenGLPolyDataMapper") == 0)
		{
			return vtkQtOpenGLPolyDataMapper::New();
		}
	}

	return NULL;
}


const char* vtkQtObjectFactory::GetVTKSourceVersion()
{
  return VTK_SOURCE_VERSION;
}



vtkObject *
vtkQtObjectFactory::vtkQtDebugWidget(QWidget* widgetParent, char* widgetName)
{
	return vtkQtOutputWindow::UseQtWidget(widgetParent, widgetName);
}


void 
vtkQtObjectFactory::SetCompileAndExecute(int oneStep)
{
	vtkQtObjectFactory::compileAndExecute = oneStep;
}


int 
vtkQtObjectFactory::GetCompileAndExecute()
{
	return vtkQtObjectFactory::compileAndExecute;
}
