// vtkQtObjectFactory.h: Schnittstelle fr die Klasse vtkQtObjectFactory.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_vtkQtObjectFactory_H__9C288733_8E23_11D4_B99B_00105AEFC335__INCLUDED_)
#define AFX_vtkQtObjectFactory_H__9C288733_8E23_11D4_B99B_00105AEFC335__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vtkObjectFactory.h>
#include <vtkWindow.h>
#include <vtkCommand.h>
#include <qgl.h>


/*
Behobene Fehler allgemein:
	vtkPolyData:				Probleme mit DeepCopy() (Achtung BuildLinks und BuildCells werden nicht aufgerufen)
	vtkLight:					Probleme bei Oeffnungswinkel des Lichtes 90-180Grad
	vtkLookupTable:				Keine Eintragsbeschraenkung von 65536 Farben
	vtkLogLookupTable:			Keine Eintragsbeschraenkung von 65536 Farben
	vtkWindowLevelLookupTable:	Keine Eintragsbeschraenkung von 65536 Farben

Behobene Fehler mit Bezug zu QT:
	vtkTexture:					Statische Casts
	vktOpenGLPolyDataMapper:	Statische Casts
*/


class vtkQtObjectFactory : public vtkObjectFactory
{
public:
	enum EventIds 
	{
		StereoOneEyeEndEvent = vtkCommand::UserEvent + 1000
	};



	/*!
	 * Erzeugt eine neue ObjectFactory, die vtk mit qt kombiniert
	 */
	static vtkQtObjectFactory*
		New();

	/*!
	 * Diese Parameter werden fuer das naechste QtWidget verwendet.
	 *
	 * Anschliessend werden diese globalen Parameter geloescht.
	 */
	static void
		SetQtBase(QWidget* widgetParent, char* widgetName = NULL, QGLWidget* widgetShared = NULL);

	/*!
	 * Diese Parameter werden fuer das naechste QtWidget verwendet.
	 *
	 * Anschliessend werden diese globalen Parameter geloescht.
	 */
	static void
		GetQtBase(QWidget* &widgetParent, char* &widgetName, QGLWidget* &widgetShared);

	/*!
	 * Transformiert ein VtkObject (vtkWindow oder vtkOutputWindow) in ein QWidget.
	 */
	static QWidget*
		QtWidget(vtkObject*);

	/*!
	 * Transformiert ein QWidget in ein VtkObject (vtkWindow oder vtkOutputWindow).
	 */
	static vtkObject*
		VtkWindow(QWidget*);

	/*!
	 * Registriert diese ObjectFactory.
	 *
	 * Die Registrierung wird nur einmal zu Beginn benoetigt
	 * und loest saemtliche bekannten Probleme mit vtk Klassen,
	 * die z.B. im Zusammenhang mit Qt auftraten.
	 */
	static void
		RegisterObjectFactory();

	static vtkObject *
		vtkQtDebugWidget(QWidget* widgetParent, char* widgetName);

    vtkTypeMacro(vtkQtObjectFactory,vtkObjectFactory);

	virtual const char* GetVTKSourceVersion();

	const char* GetDescription() { return "The MeGI Object Factory supports QT"; }

	static void SetCompileAndExecute(int oneStep);
	static int GetCompileAndExecute();


protected:  
	vtkQtObjectFactory() {};
	~vtkQtObjectFactory() {};
	vtkQtObjectFactory(const vtkQtObjectFactory&) {};
	void operator=(const vtkQtObjectFactory&) {};


protected:
	/*!
	 * Generiert entsprechene Objekte.
	 */
    virtual vtkObject*
		CreateObject(const char* vtkclassname);

	/*!
	 * Wurde die Factory bereits registriert.
	 */
	static int
		registered;

	/*!
	 * Das Parent Widget fuer das naechste vtkWindow.
	 */
	static QWidget*
		widgetParent;

	/*!
	 * Der Name fuer das naechste vtkWindow.
	 */
	static char*
		widgetName;

	/*!
	 * Mirror OpenGL images?
	 */
	static int
		mirrorOpenGL;

	/*!
	 * Improve perfromance if Compile and Execute is faster than CompileAndExecute
	 * VTK 3.2 use always CompileAndExecute
	 */
	static int
		compileAndExecute;

	/*!
	 */
	static QGLWidget*
		widgetShared;
};

#endif // !defined(AFX_vtkQtObjectFactory_H__9C288733_8E23_11D4_B99B_00105AEFC335__INCLUDED_)
